"use strict";
import { Functions as fn } from './Functions';
/**
 * The enumeration which decides what unit to use when converting to a string.
 */
export var OutputUnit;
(function (OutputUnit) {
    /**
     * This value will keep units from being displayed.
     */
    OutputUnit[OutputUnit["NONE"] = 0] = "NONE";
    /**
     * This value will ensure the unit exactly as the user entered it is used in
     * the output no matter whether the value's plurality matches the given
     * unit's plurality.
     *
     * @see [[Value.unit]]
     */
    OutputUnit[OutputUnit["GIVEN"] = 1] = "GIVEN";
    /**
     * This value will force the short versions of the unit to be used.
     *
     * @see [[Group.singularShort]]
     * @see [[Group.pluralShort]]
     */
    OutputUnit[OutputUnit["SHORT"] = 2] = "SHORT";
    /**
     * This value will force the long versions of the unit to be used.
     *
     * @see [[Group.singularLong]]
     * @see [[Group.pluralLong]]
     */
    OutputUnit[OutputUnit["LONG"] = 3] = "LONG";
})(OutputUnit = OutputUnit || (OutputUnit = {}));
/**
 * The enumeration which decides how a value will be converted to a string.
 */
export var OutputFormat;
(function (OutputFormat) {
    /**
     * The format of the user input will be used if possible.
     */
    OutputFormat[OutputFormat["GIVEN"] = 0] = "GIVEN";
    /**
     * All values will be displayed using their decimal representation.
     */
    OutputFormat[OutputFormat["NUMBER"] = 1] = "NUMBER";
    /**
     * All values will be displayed as a mixed fraction if the value is a fraction.
     * A mixed fraction has a whole number followed by a fraction where the
     * numerator is smaller than the denominator.
     *
     * @see [[Value.isFraction]]
     */
    OutputFormat[OutputFormat["MIXED"] = 2] = "MIXED";
    /**
     * All values will be displayed as an improper fraction if the value is a
     * fraction and the numerator is larger than the denoninator.
     *
     * @see [[Value.isFraction]]
     */
    OutputFormat[OutputFormat["IMPROPER"] = 3] = "IMPROPER";
})(OutputFormat = OutputFormat || (OutputFormat = {}));
/**
 * The class which converts Unitz objects to strings.
 */
export class Output {
    /**
     * Creates a new instance of Output with an optional set of options to
     * override the default values.
     *
     * @param input The options to apply to the new instance.
     */
    constructor(input) {
        /**
         * The option that specifies which units are chosen.
         */
        this.unit = OutputUnit.GIVEN;
        /**
         * The option that specifies how values are displayed.
         */
        this.format = OutputFormat.GIVEN;
        /**
         * Whether or not a unit should be displayed for the minimum and maximum of a
         * range when they have the same group.
         */
        this.repeatUnit = false;
        /**
         * The spacing used between the value and the unit.
         */
        this.unitSpacer = '';
        /**
         * The spacing used between the minimum and maximum values in a range.
         */
        this.rangeSpacer = ' - ';
        /**
         * The spacing used to separate the numerator and denominator of a fraction.
         */
        this.fractionSpacer = '/';
        /**
         * The spacing used to seperate a mixed number from the fraction.
         */
        this.mixedSpacer = ' ';
        /**
         * The spacing used to separate a unit and the rate unit.
         */
        this.rateSpacer = '/';
        /**
         * The delimiter used to separate ranges.
         */
        this.delimiter = ', ';
        /**
         * An option used to restrict numbers from displaying large decimal numbers.
         * When this value is set to -1 numbers are displayed fully. If the value is
         * set to zero all numbers will be truncated to the whole version.
         */
        this.significant = -1;
        if (fn.isDefined(input)) {
            this.set(input);
        }
    }
    /**
     * Overrides values in this instance with ones specified in input.
     *
     * @param input The values to override.
     * @return The reference to this instance.
     */
    set(input) {
        this.unit = fn.coalesce(input.unit, this.unit);
        this.format = fn.coalesce(input.format, this.format);
        this.repeatUnit = fn.coalesce(input.repeatUnit, this.repeatUnit);
        this.unitSpacer = fn.coalesce(input.unitSpacer, this.unitSpacer);
        this.rangeSpacer = fn.coalesce(input.rangeSpacer, this.rangeSpacer);
        this.fractionSpacer = fn.coalesce(input.fractionSpacer, this.fractionSpacer);
        this.mixedSpacer = fn.coalesce(input.mixedSpacer, this.mixedSpacer);
        this.rateSpacer = fn.coalesce(input.rateSpacer, this.rateSpacer);
        this.delimiter = fn.coalesce(input.delimiter, this.delimiter);
        this.significant = fn.coalesce(input.significant, this.significant);
        return this;
    }
    /**
     * Returns an Output instance which matches the desired options. If no options
     * are specified the reference to this instance is returned. If the options
     * are already an instance of Output its returned. If options are specified
     * a new instance is created with the options of this instance, and the given
     * options applied with [[Output.set]].
     *
     * @param input The options desired.
     * @return An instance of this class which matches the desired options.
     */
    extend(input) {
        let extended = this;
        if (fn.isDefined(input)) {
            if (input instanceof Output) {
                extended = input;
            }
            else {
                extended = new Output(this);
                extended.set(input);
            }
        }
        return extended;
    }
    /**
     * Converts the list of ranges to a string. If a range is not valid it is
     * skipped.
     *
     * @param ranges The list of ranges to convert.
     * @return The string representation of the input.
     */
    ranges(ranges) {
        let out = '';
        for (let i = 0; i < ranges.length; i++) {
            let range = ranges[i];
            if (range.isValid) {
                if (out.length) {
                    out += this.delimiter;
                }
                out += this.range(range);
            }
        }
        return out;
    }
    /**
     * Converts the range to a string.
     *
     * @param ranges The range to convert.
     * @return The string representation of the input.
     */
    range(range) {
        let out = '';
        if (!range.isValid) {
            // nothing
        }
        else if (range.isFixed) {
            out += this.value(range.min);
        }
        else {
            let minUnit = this.repeatUnit || range.min.unit !== range.max.unit;
            out += this.value(range.min, minUnit);
            out += this.rangeSpacer;
            out += this.value(range.max);
        }
        return out;
    }
    /**
     * Converts the value to the string optionally showing or hiding the unit.
     *
     * @param value The value to convert.
     * @param showUnit Whether or not the unit should be added to the string.
     * @return The string representation of the input.
     */
    value(value, showUnit = true) {
        let out = '';
        if (!value.isValid) {
        }
        else if (this.isFraction(value)) {
            if (this.isMixed(value)) {
                out += value.mixedWhole;
                out += this.mixedSpacer;
                out += value.mixedNum;
                out += this.fractionSpacer;
                out += value.den;
            }
            else {
                out += value.num;
                out += this.fractionSpacer;
                out += value.den;
            }
        }
        else {
            out += this.number(value.value);
        }
        if (showUnit && this.unit !== OutputUnit.NONE && value.isValid) {
            out += this.unitSpacer;
            out += this.units(value);
        }
        return out;
    }
    /**
     * Generates a full unit string including the rate unit if it exists.
     *
     * @param value The value to generate a unit for.
     * @return The units string representation.
     */
    units(value) {
        let out = '';
        out += this.group(value.value, value.unit, value.group);
        if (value.rateGroup) {
            out += this.rateSpacer;
            out += this.group(value.value, value.rate, value.rateGroup);
        }
        return out;
    }
    /**
     * Generates a unit string given the value, the current unit, and its group.
     *
     * @param value The value to generate a unit for.
     * @param unit The unit to potentially use.
     * @param group The group of the unit.
     * @return The unit determined based on the options.
     */
    group(value, unit, group) {
        if (this.isLongUnit(group)) {
            return fn.isSingular(value) ? group.singularLong : group.pluralLong;
        }
        else if (this.isShortUnit(group) || (group && group.dynamic)) {
            return fn.isSingular(value) ? group.singularShort : group.pluralShort;
        }
        return unit;
    }
    /**
     * Converts the number to a string.
     *
     * @param x The number to convert.
     * @return The string representation of the input.
     */
    number(x) {
        let valueString = x + '';
        if (this.significant >= 0 && valueString !== '0') {
            let valueSignificant = x
                .toFixed(this.significant)
                .replace(/0*$/, '')
                .replace(/\.$/, '');
            return valueSignificant.length < valueString.length ? valueSignificant : valueString;
        }
        return valueString;
    }
    /**
     * Determines whether the value should be displayed as a fraction.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a fraction, otherwise false.
     */
    isFraction(value) {
        return value.isFraction && this.format !== OutputFormat.NUMBER;
    }
    /**
     * Determines whether the value should be displayed as a number.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a number, otherwise false.
     */
    isNumber(value) {
        return value.isValid && !this.isFraction(value);
    }
    /**
     * Determines whether the value should be displayed as a mixed fraction. This
     * assumes [[Output.isFraction]] was already checked and returned true.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a mixed fraction, otherwise false.
     */
    isMixed(value) {
        return value.mixedWhole !== 0 && this.format !== OutputFormat.IMPROPER;
    }
    /**
     * Determines whether the short unit should be displayed.
     *
     * @param group The group of the unit.
     * @return True if the short unit should be displayed, otherwise false.
     */
    isShortUnit(group) {
        return group && this.unit === OutputUnit.SHORT;
    }
    /**
     * Determines whether the long unit should be displayed.
     *
     * @param group The group of the unit.
     * @return True if the short unit should be displayed, otherwise false.
     */
    isLongUnit(group) {
        return group && this.unit === OutputUnit.LONG;
    }
}
//# sourceMappingURL=Output.js.map