"use strict";
import { Functions as fn } from './Functions';
import { Core } from './Core';
import { Range } from './Range';
import { Value } from './Value';
import { Base } from './Base';
import { Translations } from './Translations';
import { Rates } from './Rates';
/**
 * The class which takes user input and parses it to specific structures.
 */
export class Parse {
    /**
     * Parses user input into a [[Base]] instance.
     *
     * @param input The input to parse into a Base.
     * @return The instance parsed from the input.
     */
    static base(input) {
        if (input instanceof Base) {
            return input;
        }
        return new Base(input);
    }
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static ranges(input) {
        if (fn.isArray(input)) {
            return this.rangesFromArray(input);
        }
        else if (fn.isString(input)) {
            return this.rangesFromString(input);
        }
        else if (fn.isRangeDefinition(input)) {
            return this.rangesFromArray([input]);
        }
        else if (fn.isValueDefinition(input)) {
            return this.rangesFromArray([input]);
        }
        return [];
    }
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static rangesFromArray(input) {
        let ranges = [];
        for (let i = 0; i < input.length; i++) {
            let range = this.range(input[i]);
            ranges.push(range);
        }
        return ranges;
    }
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static rangesFromString(input) {
        let ranges = input.split(this.REGEX_LIST);
        return this.rangesFromArray(ranges);
    }
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static range(input) {
        if (fn.isString(input)) {
            return this.rangeFromString(input);
        }
        else if (fn.isRangeDefinition(input)) {
            let min = this.value(input.min);
            let max = this.value(input.max);
            return new Range(min, max);
        }
        else if (fn.isValueDefinition(input)) {
            let value = this.valueFromValue(input);
            return new Range(value, value);
        }
        return Range.INVALID;
    }
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static rangeFromString(input) {
        let matches = this.REGEX_RANGE.exec(input);
        if (!matches) {
            let fixed = this.valueFromString(input);
            return new Range(fixed, fixed);
        }
        let minInput = matches[1];
        let maxInput = matches[2];
        let minParsed = this.input(minInput);
        let maxParsed = this.input(maxInput);
        if (!minParsed || !maxParsed) {
            return Range.INVALID;
        }
        let minUnit = minParsed.unit || maxParsed.unit;
        let maxUnit = maxParsed.unit || minParsed.unit;
        let minRate = minParsed.rate || maxParsed.rate;
        let maxRate = maxParsed.rate || minParsed.rate;
        let min = this.valueFromResult(minParsed, minUnit, minRate);
        let max = this.valueFromResult(maxParsed, maxUnit, maxRate);
        return new Range(min, max);
    }
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static value(input) {
        if (fn.isString(input)) {
            return this.valueFromString(input);
        }
        else if (fn.isValueDefinition(input)) {
            return this.valueFromValue(input);
        }
        return Value.INVALID;
    }
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static valueFromValue(input) {
        let givenValue = fn.isDefined(input.value) ? input.value : 1;
        let num = fn.isDefined(input.num) ? input.num : givenValue;
        let den = fn.isDefined(input.den) ? input.den : 1;
        let parsedValue = fn.isDefined(input.value) ? input.value : num / den;
        let unit = input.unit || '';
        let rate = input.rate || '';
        let group = Core.getGroup(unit);
        let rateGroup = Core.getGroup(rate);
        return new Value(parsedValue, num, den, unit, group, rate, rateGroup);
    }
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static valueFromString(input) {
        let translated = Translations.translate(input);
        let parsed = this.input(translated);
        return parsed ? this.valueFromResult(parsed, parsed.unit, parsed.rate) : Value.INVALID;
    }
    /**
     * Parses user input into a [[Value]].
     *
     * @param result The already parsed input.
     * @param unit The unit parsed from the input.
     * @return The instance parsed from the input.
     */
    static valueFromResult(result, unit, rateUnit) {
        let group = Core.getGroup(unit);
        let rateGroup = Core.getGroup(rateUnit);
        return new Value(result.value, result.valueNum, result.valueDen, unit, group, rateUnit, rateGroup);
    }
    /**
     * Parses user input into a [[ParseResult]]. If the input is not valid null
     * is returned.
     *
     * *Examples:*
     * - 1tsp
     * - 1 tsp
     * - 1/2 tsp
     * - 1 1/2 tsp
     * - -2 cups
     * - 2.35"
     *
     * @param input The string to parse a value and unit from.
     * @return The result of the parsing.
     */
    static input(input) {
        let matches = this.REGEX_PARSE.exec(input);
        let whole = parseInt(matches[1]);
        let hasWhole = isFinite(whole);
        let sign = matches[1].charAt(0) === '-' ? -1 : 1;
        let num = parseInt(matches[3]);
        let den = parseInt(matches[5]);
        let decimal = matches[6];
        let hasDecimal = isFinite(parseFloat(decimal));
        let { unit, rate } = this.unit(fn.trim(matches[7]));
        if (!hasWhole && hasDecimal) {
            whole = 0;
            hasWhole = true;
        }
        if (!hasWhole && !unit) {
            return null;
        }
        let value = 1;
        let valueDen = 1;
        let valueNum = 1;
        if (hasWhole) {
            value = whole;
            valueNum = whole;
            if (isFinite(den)) {
                valueDen = den;
                if (isFinite(num)) {
                    value += (num / den) * sign;
                    valueNum *= den;
                    valueNum += num;
                }
                else {
                    value /= den;
                }
            }
            else if (hasDecimal) {
                let remainder = parseFloat('0.' + decimal);
                value += remainder * sign;
                valueNum += remainder;
            }
            valueNum *= sign;
        }
        return { value, valueNum, valueDen, num, den, unit, rate };
    }
    /**
     * Parses unit input into a [[Rate]].
     *
     * *Examples:*
     * - m/s
     * - miles per hour
     * - mph
     *
     * @param input The string to parse a unit from.
     * @return The result of the parsing.
     */
    static unit(input) {
        let rate = Rates.get(input);
        if (!rate) {
            let units = input.split(this.REGEX_UNIT);
            rate = {
                unit: units[0] ? fn.trim(units[0]).replace(/\.$/, '') : '',
                rate: units[2] ? fn.trim(units[2]).replace(/\.$/, '') : ''
            };
        }
        return rate;
    }
}
/**
 * The regular expression used to split up a string into multiple ranges.
 */
Parse.REGEX_LIST = /\s*,\s*/;
/**
 * The regular expression used to split up a range string to determine the min
 * and maximum values.
 */
Parse.REGEX_RANGE = /\s*(-?[^-]+)-(.+)/;
/**
 * The regular expression used to parse a value number or fraction and
 * possible unit from a string.
 */
Parse.REGEX_PARSE = /^\s*(-?\d*)(\s+(\d+))?(\s*\/\s*(\d+)|\.(\d+)|)\s*(.*)\s*$/i;
/**
 * The regular expression used to split up a unit from a rateUnit.
 */
Parse.REGEX_UNIT = /\s*(\/|\s+per\s+)\s*/i;
//# sourceMappingURL=Parse.js.map