"use strict";
import { Functions as fn } from './Functions';
import { Value } from './Value';
import { Core } from './Core';
/**
 * A pair of minimum and maximum values. A range can be fixed which means the
 * minimum and maximum are equivalent - in which case the range behaves like
 * a [[Value]].
 */
export class Range {
    /**
     * Creates a new instance of Range given the minimum and maximum values.
     *
     * @param min The minimum value for the range.
     * @param max The maximum value for the range.
     */
    constructor(min, max) {
        this.min = min.value < max.value ? min : max;
        this.max = max.value > min.value ? max : min;
    }
    /**
     * True if the min and max are both valid.
     */
    get isValid() {
        return this.min.isValid && this.max.isValid;
    }
    /**
     * True if the min or max are a fraction.
     */
    get isFraction() {
        return this.min.isFraction || this.max.isFraction;
    }
    /**
     * True if the min and max are decimal.
     */
    get isDecimal() {
        return this.min.isDecimal && this.max.isDecimal;
    }
    /**
     * True if the min and max are not the same value.
     */
    get isRange() {
        return this.min.value !== this.max.value;
    }
    /**
     * True if the min and max are the same value.
     */
    get isFixed() {
        return this.min.value === this.max.value;
    }
    /**
     * True if the min and max are both equal to zero.
     */
    get isZero() {
        return this.min.isZero && this.max.isZero;
    }
    /**
     * True if the min and max are both singular (1 or -1).
     */
    get isSingular() {
        return this.min.isSingular && this.max.isSingular;
    }
    /**
     * True if one of min and max are rates.
     */
    get isRate() {
        return this.min.isRate && this.max.isRate;
    }
    /**
     * The average number between the min and max.
     */
    get average() {
        return (this.min.value + this.max.value) * 0.5;
    }
    /**
     * The minimum value of this range.
     */
    get value() {
        return this.min.value;
    }
    /**
     * The minimum value of this range.
     */
    get minimum() {
        return this.min.value;
    }
    /**
     * The maximum value of this range.
     */
    get maximum() {
        return this.max.value;
    }
    /**
     * The unit which identifies the group of the minimum value or `null` if the
     * minimum value does not have a group.
     */
    get unit() {
        return this.min.group ? this.min.group.unit : null;
    }
    /**
     * Determines if the given range matches this range enough to allow a
     * simple mathematical operation between the two ranges.
     *
     * @param range The range to test.
     * @return True if the groups of the given range match this range.
     */
    isExactMatch(range) {
        return this.min.group === range.min.group &&
            this.max.group === range.max.group &&
            this.min.rateGroup === range.min.rateGroup &&
            this.max.rateGroup === range.max.rateGroup;
    }
    /**
     * Determines if the given range matches this range enough to allow a
     * complex mathematical operation between the two ranges.
     *
     * @param min The minimum of the range to test.
     * @param max The maximum of the range to test.
     * @return True if the min and max have compatible values.
     */
    isMatch(min, max) {
        return this.min.isMatch(min) && this.max.isMatch(max);
    }
    /**
     * @return A range which has the min and max converted to their preferred units.
     * @see [[Value.preferred]]
     */
    preferred() {
        let min = this.min.preferred();
        let max = this.max.preferred();
        return new Range(min, max);
    }
    /**
     * @return A range which has only positive values. If the range is entirely
     *  negative then `null` is returned.
     */
    positive() {
        let minNegative = this.min.value < 0;
        let maxNegative = this.max.value < 0;
        if (maxNegative) {
            return null;
        }
        let min = minNegative ? this.min.zero() : this.min.copy();
        let max = this.max.copy();
        return new Range(min, max);
    }
    /**
     * @return A range which has only negative values. If the range is entirely
     *  positive then `null` is returned.
     */
    negative() {
        let minPositive = this.min.value >= 0;
        let maxPositive = this.max.value >= 0;
        if (minPositive) {
            return null;
        }
        let min = this.min.copy();
        let max = maxPositive ? this.max.zero() : this.max.copy();
        return new Range(min, max);
    }
    /**
     * @return A range which has a non-zero min and max. If both are equial to
     *  zero then `null` is returned.
     */
    nonzero() {
        let minZero = fn.isZero(this.min.value);
        let maxZero = fn.isZero(this.max.value);
        if (minZero && maxZero) {
            return null;
        }
        let min = this.min.copy();
        let max = this.max.copy();
        return new Range(min, max);
    }
    /**
     * @return A range with only the maximum value from this range.
     */
    maxd() {
        let fixed = this.max.copy();
        return new Range(fixed, fixed);
    }
    /**
     * @return A range with only the minimum value from this range.
     */
    mind() {
        let fixed = this.min.copy();
        return new Range(fixed, fixed);
    }
    /**
     * Creates a range with with units that best represent the values. This may
     * cause the minimum and maximum values to have different units.
     *
     * @param transform Options to control which units and values are acceptable.
     * @param forOutput The output options that should be used to determine which
     *  value & unit is best.
     * @return A new range.
     * @see [[Value.normalize]]
     */
    normalize(transform, forOutput) {
        let min = this.min.normalize(transform, forOutput);
        let max = this.max.normalize(transform, forOutput);
        return new Range(min, max);
    }
    /**
     * Adds this range and a given range (optionally scaled by a factor) together.
     *
     * @param addend The range to add to this instance.
     * @param scale The factor to multiply the addend by when added it to this
     *  instance.
     * @return a new range.
     * @see [[Value.add]]
     */
    add(addend, scale = 1) {
        let min = this.min.add(addend.min, scale);
        let max = this.max.add(addend.max, scale);
        return new Range(min, max);
    }
    /**
     * Subtracts a given range (optionally scaled by a factor) from this range.
     *
     * @param subtrahend The range to remove from this instance.
     * @param scale The factor to multiply the subtrahend by when subtracting it
     *  from this instance.
     * @return A new range.
     * @see [[Value.sub]]
     */
    sub(subtrahend, scale = 1) {
        let min = this.min.sub(subtrahend.min, scale);
        let max = this.max.sub(subtrahend.max, scale);
        return new Range(min, max);
    }
    /**
     * Multiplies this range by a scalar factor.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.scale]]
     */
    scale(scale) {
        let min = this.min.scale(scale);
        let max = this.max.scale(scale);
        return new Range(min, max);
    }
    /**
     * Multiplies this range by a scale value.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.mul]]
     */
    mul(scale) {
        let min = this.min.mul(scale);
        let max = this.max.mul(scale);
        return new Range(min, max);
    }
    /**
     * Returns a range which is coerced into being represented by fractions if a
     * valid fraction can be determined from the units valid denominators.
     *
     * @return A new range if the minimum and maximum are not fractions, otherwise
     *  the reference to this range is returned.
     * @see [[Value.fractioned]]
     */
    fractioned() {
        if (this.min.isFraction && this.max.isFraction) {
            return this;
        }
        let min = this.min.fractioned();
        let max = this.max.fractioned();
        return new Range(min, max);
    }
    /**
     * Returns a range which has any fraction values converted to numbers.
     *
     * @return A new range if the mimimum or maximum are fractions, otherwise the
     *  the reference to this range is returned.
     * @see [[Value.numbered]]
     */
    numbered() {
        if (!this.min.isFraction && !this.max.isFraction) {
            return this;
        }
        let min = this.min.numbered();
        let max = this.max.numbered();
        return new Range(min, max);
    }
    /**
     * Converts this range to a string with the given output options taking into
     * account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    output(options) {
        let output = Core.globalOutput.extend(options);
        return output.range(this);
    }
    /**
     * Creates a fixed range from a given value. A fixed range behaves essentially
     * as a value since the minimum and maximum are equivalent.
     *
     * @param fixed The value to be used as the min and max of the range.
     * @return A new fixed range.
     */
    static fromFixed(fixed) {
        return new Range(fixed, fixed);
    }
}
/**
 * A range instance which contains invalid values.
 */
Range.INVALID = new Range(Value.INVALID, Value.INVALID);
//# sourceMappingURL=Range.js.map