"use strict";
import { Functions as fn } from './Functions';
/**
 * The enumeration which decides what value in a range should be used when
 * sorting between ranges with differing minimum and maximum values.
 */
export var SortType;
(function (SortType) {
    /**
     * This value will use the minimum of the ranges to sort by.
     */
    SortType[SortType["MIN"] = 0] = "MIN";
    /**
     * This value will use the maximum of the ranges to sort by.
     */
    SortType[SortType["MAX"] = 1] = "MAX";
    /**
     * This value will use the average of the ranges to sort by.
     */
    SortType[SortType["AVERAGE"] = 2] = "AVERAGE";
})(SortType = SortType || (SortType = {}));
/**
 * The class which determines how to sort ranges.
 */
export class Sort {
    /**
     * Creates a new instance of Sort with an optional set of options to override
     * the default values.
     *
     * @param input The options to apply to the new instance.
     */
    constructor(input) {
        /**
         * If the ranges should be in ascending order (small values followed by large
         * values). The default value is in descending order.
         */
        this.ascending = false;
        /**
         * How ranges should be compared when the minimum and maximum values differ.
         */
        this.type = SortType.MAX;
        /**
         * This object describes how ranges of different classes should be sorted by
         * given each class a priority. If a class is not defined here the priority
         * assumed is zero.
         */
        this.classes = {};
        if (fn.isDefined(input)) {
            this.set(input);
        }
    }
    /**
     * Overrides values in this instance ith ones specified in the input. If class
     * sorting options are specified they are merged into this instance as opposed
     * to a complete overwrite.
     *
     * @param input The values to override.
     * @return The reference to this instance.
     */
    set(input) {
        this.ascending = fn.coalesce(input.ascending, this.ascending);
        this.type = fn.coalesce(input.type, this.type);
        if (fn.isDefined(input.classes)) {
            for (let className in input.classes) {
                this.classes[className] = input.classes[className];
            }
        }
        return this;
    }
    /**
     * Returns a Sort instance which matches the desired options. If no options
     * are specified the reference to this instance is returned. If the options
     * are already an instance of Sort its returned. If options are specified
     * a new instance is created with the options of this instance, and the given
     * options applied with [[Sort.set]].
     *
     * @param input The options desired.
     * @return An instance of this class which matches the desired options.
     */
    extend(input) {
        let extended = this;
        if (fn.isDefined(input)) {
            if (input instanceof Sort) {
                extended = input;
            }
            else {
                extended = new Sort(this);
                extended.set(input);
            }
        }
        return extended;
    }
    /**
     * Returns a function which can sort ranges based on the options in this
     * instance. Comparison is first done by class, and followed by type.
     */
    getSorter() {
        return (a, b) => {
            let d = this.getClassComparison(a, b);
            if (d === 0) {
                switch (this.type) {
                    case SortType.MIN:
                        d = this.getMinComparison(a, b);
                        break;
                    case SortType.MAX:
                        d = this.getMaxComparison(a, b);
                        break;
                    case SortType.AVERAGE:
                        d = this.getAverageComparison(a, b);
                        break;
                }
            }
            return this.ascending ? d : -d;
        };
    }
    /**
     * A sort function between two ranges which look at the range minimums.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    getMinComparison(a, b) {
        return fn.sign(a.min.classScaled - b.min.classScaled);
    }
    /**
     * A sort function between two ranges which look at the range maximums.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    getMaxComparison(a, b) {
        return fn.sign(a.max.classScaled - b.max.classScaled);
    }
    /**
     * A sort function between two ranges which look at the range averages.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    getAverageComparison(a, b) {
        let avg = (a.min.classScaled + a.max.classScaled) * 0.5;
        let bvg = (b.min.classScaled + b.max.classScaled) * 0.5;
        return fn.sign(avg - bvg);
    }
    /**
     * A sort function between two ranges which look at the range classes.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    getClassComparison(a, b) {
        let ag = a.min.group ? 1 : -1;
        let bg = b.min.group ? 1 : -1;
        if (ag !== bg) {
            return ag - bg;
        }
        let ac = this.classes[a.min.group.parent.name] || 0;
        let bc = this.classes[b.min.group.parent.name] || 0;
        return ac - bc;
    }
}
//# sourceMappingURL=Sort.js.map