"use strict";
import { Functions as fn } from './Functions';
import { Core } from './Core';
import { Parse } from './Parse';
import { Range } from './Range';
import { Value } from './Value';
/**
 * Takes user input and returns a new instance of [Base].
 */
export function uz(input) {
    return new Base(input);
}
/**
 * The main class which contains a list of ranges and the user input.
 */
var Base = (function () {
    /**
     * Creates a new instance of Base given some user input to parse or an
     * existing list of ranges to use instead.
     *
     * @param input The input to parse if ranges is not provided.
     * @param ranges The already parsed ranges to use for this instance.
     */
    function Base(input, ranges) {
        this.input = input;
        this.ranges = ranges || Parse.ranges(input);
    }
    /**
     * Scales the ranges in this instance by the given factor and returns a
     * new instance.
     *
     * *For example:*
     * ```javascript
     * uz('1c, 2.3m').scale(2); // '2c, 4.6m'
     * ```
     *
     * @param amount The factor to scale the ranges in this instance by.
     * @return A new instance.
     * @see [[Range.scale]]
     * @see [[Base.mutate]]
     */
    Base.prototype.scale = function (amount) {
        return this.mutate(function (r) { return r.scale(amount); });
    };
    /**
     * Scales the ranges in this instance by the given value and returns a
     * new instance.
     *
     * *For example:*
     * ```javascript
     * uz('1c, 3/5m').scale(Value.fromFraction(2, 3)); // '2/3c, 6/15m'
     * ```
     *
     * @param amount The value to scale the ranges in this instance by.
     * @return A new instance.
     * @see [[Range.mul]]
     * @see [[Base.mutate]]
     */
    Base.prototype.mul = function (amount) {
        return this.mutate(function (r) { return r.mul(amount); });
    };
    // 1c, 3m SCALE TO 1/2c = 1/2c, 1.5m
    /**
     * Scales the ranges in this instance up to some value with a unit and returns
     * a new instance. Because this instance might contain ranges, a rangeDelta
     * can be specified to instruct on which value (min or max) to use when
     * calculating how much to scale by.
     *
     * *For example:*
     * ```javascript
     * uz('1m, 2 - 3c').scaleTo('6c'); // '2m, 4 - 6c'
     * uz('1m, 2 - 3c').scaleTo('6c', 0); // '3m, 6 - 9c'
     * uz('1m, 2 - 3c').scaleTo('6c', 0.5); // '2.4m, 4.8 - 6c'
     * ```
     *
     * @param unitValue A value & unit pair to scale the ranges in this instance to.
     * @param rangeDelta When this instance contains ranges this value instructs
     *  how the scale factor is calculated. A value of 0 means it looks at the
     *  minimum, 1 is the maximum, and 0.5 is the average.
     * @return A new instance.
     * @see [[Base.getScaleTo]]
     * @see [[Base.scale]]
     */
    Base.prototype.scaleTo = function (unitValue, rangeDelta) {
        if (rangeDelta === void 0) { rangeDelta = 1.0; }
        return this.scale(this.getScaleTo(unitValue, rangeDelta));
    };
    /**
     * Changes the units used on each of the ranges in this instance to the
     * preferred unit for each group.
     *
     * *For example:*
     * ```javascript
     * uz('5 kilos').preferred(); // '5 kg'
     * ```
     *
     * @return A new instance.
     * @see [[Core.setPreferred]]
     * @see [[Range.preferred]]
     * @see [[Base.mutate]]
     */
    Base.prototype.preferred = function () {
        return this.mutate(function (r) { return r.preferred(); });
    };
    /**
     * Drops negative ranges and modifies partially negative ranges so that all
     * values are greater than or equal to zero.
     *
     * *For example:*
     * ```javascript
     * uz('0c, 2tbsp, -4tbsp').positive(); // '0c, 2tbsp'
     * uz('-2 - 3 in').positive(); // '0 - 3in'
     * ```
     *
     * @return A new instance.
     * @see [[Range.positive]]
     * @see [[Base.mutate]]
     */
    Base.prototype.positive = function () {
        return this.mutate(function (r) { return r.positive(); });
    };
    /**
     * Drops positive ranges and modifies partially positive ranges so that all
     * values are less than zero.
     *
     * *For example:*
     * ```javascript
     * uz('0c, 2tbsp, -4tbsp').negative(); // '-4tbsp'
     * uz('-2 - 3 in').negative(); // '-2 - 0in'
     * ```
     *
     * @return A new instance.
     * @see [[Range.negative]]
     * @see [[Base.mutate]]
     */
    Base.prototype.negative = function () {
        return this.mutate(function (r) { return r.negative(); });
    };
    /**
     * Drops ranges that are equal to zero.
     *
     * *For example:*
     * ```javascript
     * uz('0c, 2tbsp').negative(); // '2tbsp'
     * ```
     *
     * @return A new instance.
     * @see [[Range.nonzero]]
     * @see [[Base.mutate]]
     */
    Base.prototype.nonzero = function () {
        return this.mutate(function (r) { return r.nonzero(); });
    };
    /**
     * Converts each range to fractions if a denominator for the specified units
     * yields a fraction close enough to the original value.
     *
     * *For example:*
     * ```javascript
     * uz('1/2 cup').fractions(); // '1/2 cup'
     * uz('0.3cm').fractions(); // '3/10 cm'
     * uz('0.33 decades').fractions(); // '0.33 decades' closest is 3/10 but that's not close enough
     * ```
     *
     * @return A new instance.
     * @see [[Range.fractioned]]
     * @see [[Base.mutate]]
     */
    Base.prototype.fractions = function () {
        return this.mutate(function (r) { return r.fractioned(); });
    };
    /**
     * Converts each range to numbers if they are fractions.
     *
     * *For example:*
     * ```javascript
     * uz('1/2 cup').fractions(); // '0.5 cup'
     * uz('0.3cm').fractions(); // '0.3 cm'
     * ```
     *
     * @return A new instance.
     * @see [[Range.numbered]]
     * @see [[Base.mutate]]
     */
    Base.prototype.numbers = function () {
        return this.mutate(function (r) { return r.numbered(); });
    };
    /**
     * Flattens any ranges to their maximum values.
     *
     * *For example:*
     * ```javascript
     * uz('1 - 3c, 5m').max(); // '3c, 5m'
     * ```
     *
     * @return A new instance or this if this instance has no ranges.
     * @see [[Range.maxd]]
     * @see [[Base.mutate]]
     */
    Base.prototype.max = function () {
        return this.hasRanges ? this.mutate(function (r) { return r.maxd(); }) : this;
    };
    /**
     * Flattens any ranges to their minimum values.
     *
     * *For example:*
     * ```javascript
     * uz('1 - 3c, 5m').max(); // '1c, 5m'
     * ```
     *
     * @return A new instance or this if this instance has no ranges.
     * @see [[Range.mind]]
     * @see [[Base.mutate]]
     */
    Base.prototype.min = function () {
        return this.hasRanges ? this.mutate(function (r) { return r.mind(); }) : this;
    };
    /**
     * Converts each range to units that best represent the value.
     *
     * *For example:*
     * ```javascript
     * uz('1.5pt, 12in, 3.14159rad').normalize(); // '3c, 1ft, 180deg'
     * ```
     *
     * @param options Options to control which units and values are acceptable.
     * @param forOutput The output options that should be used to determine which
     *  value & unit is best.
     * @return A new instance.
     * @see [[Transform]]
     * @see [[Output]]
     * @see [[Core.isMoreNormal]]
     * @see [[Core.globalTransform]]
     * @see [[Core.globalOutput]]
     * @see [[Range.normalize]]
     * @see [[Base.mutate]]
     */
    Base.prototype.normalize = function (options, forOutput) {
        var output = Core.globalOutput.extend(forOutput);
        var transform = Core.globalTransform.extend(options);
        return this.mutate(function (r) { return r.normalize(transform, output); });
    };
    /**
     * Joins all ranges of the same classes together and uses the largest unit
     * to represent the sum for the class.
     *
     * *For example:*
     * ```javascript
     * uz('1c, 1pt').compact(); // '1.5pt'
     * ```
     *
     * @param options Options to control which units and values are acceptable.
     * @return A new instance.
     * @see [[Transform]]
     * @see [[Core.globalTransform]]
     */
    Base.prototype.compact = function (options) {
        var ranges = this.ranges.slice();
        var compacted = [];
        var transform = Core.globalTransform.extend(options);
        // Largest ranges first
        ranges.sort(function (a, b) {
            return b.max.classScaled - a.max.classScaled;
        });
        for (var i = 0; i < ranges.length; i++) {
            var a = ranges[i];
            var min = a.min;
            var max = a.max;
            for (var k = ranges.length - 1; k > i; k--) {
                var b = ranges[k];
                if (a.isMatch(b.min, b.max)) {
                    min = min.add(b.min.convertToValue(min.group, min.rateGroup));
                    max = max.add(b.max.convertToValue(max.group, max.rateGroup));
                    ranges.splice(k, 1);
                }
            }
            var sum = new Range(min, max);
            if (transform.isValidRange(sum)) {
                compacted.push(sum);
            }
        }
        return new Base(this.input, compacted);
    };
    /**
     * Joins all ranges of the same classes together and then separates them
     * into whole number ranges for better readability.
     *
     * *For example:*
     * ```javascript
     * uz('1.5pt').expand(); // '1pt, 1c'
     * uz('53in').expand(); // '4ft, 5in'
     * uz('2ft, 29in').expand(); // '4ft, 5in'
     * uz('6543mm').expand(); // '6 m, 54 cm, 3 mm'
     * ```
     *
     * @param options Options to control which units and values are acceptable.
     * @return A new instance.
     * @see [[Transform]]
     * @see [[Core.globalTransform]]
     */
    Base.prototype.expand = function (options) {
        var transform = Core.globalTransform.extend(options);
        var compacted = this.compact(transform);
        var ranges = compacted.ranges;
        var expanded = [];
        var _loop_1 = function (i) {
            var range = ranges[i];
            var value = transform.convertWithMax ? range.max : range.min;
            var valueGroup = value.group;
            var valueSign = fn.sign(value.value);
            var valueRate = value.rateGroup;
            if (valueGroup) {
                valueGroup.matches(transform, true, function (group) {
                    if (!fn.isZero(value.value)) {
                        var transformed = value.convertToValue(group, valueRate);
                        if (group.isBase) {
                            value = value.zero();
                            expanded.push(Range.fromFixed(transformed));
                        }
                        else if (fn.abs(transformed.value) >= 1 && fn.sign(transformed.value) === valueSign) {
                            var truncated = transformed.truncated();
                            value = value.sub(truncated.convertToValue(valueGroup, valueRate));
                            expanded.push(Range.fromFixed(truncated));
                        }
                    }
                });
            }
            else {
                expanded.push(range);
            }
        };
        for (var i = 0; i < ranges.length; i++) {
            _loop_1(i);
        }
        return new Base(this.input, expanded);
    };
    /**
     * Adds the ranges of this instance and the given input together. When the
     * ranges use the same units they are added together, otherwise they are
     * added to the end of the range list.
     *
     * *For example:*
     * ```javascript
     * uz('1pt').add('2pt, 1c'); // '3pt, 1c'
     * uz('1pt').add('2pt, 1c', 2); // '5pt, 2c'
     * ```
     *
     * @param input An instance or input which can be parsed into an instance.
     * @param scale A number to multiple the input by when adding it to this instance.
     * @return A new instance.
     * @see [[Base.operate]]
     * @see [[Range.add]]
     * @see [[Range.scale]]
     */
    Base.prototype.add = function (input, scale) {
        if (scale === void 0) { scale = 1; }
        return this.operate(input, function (a, b) { return a.add(b, scale); }, function (a) { return a.scale(scale); });
    };
    /**
     * Subtracts the given input from the ranges of this instance. When the ranges
     * use the same units they are subtracted, otherwise they are added to the
     * end of the range list and negated.
     *
     * *For example:*
     * ```javascript
     * uz('3pt').sub('2pt, 1c'); // '1pt, -1c'
     * uz('1pt').add('2pt, 1c', 2); // '-3pt, -2c'
     * ```
     *
     * @param input An instance or input which can be parsed into an instance.
     * @param scale A number to multiple the input by when subtracting it from this instance.
     * @return A new instance.
     * @see [[Base.operate]]
     * @see [[Range.sub]]
     * @see [[Range.scale]]
     */
    Base.prototype.sub = function (input, scale) {
        if (scale === void 0) { scale = 1; }
        return this.operate(input, function (a, b) { return a.sub(b, scale); }, function (a) { return a.scale(-scale); });
    };
    /**
     * Subtracts the given input from the ranges of this instance. When the ranges
     * use the same units they are subtracted, otherwise they are added to the
     * end of the range list and negated.
     *
     * *For example:*
     * ```javascript
     * uz('3pt').sub('2pt, 1c'); // '1pt, -1c'
     * uz('1pt').add('2pt, 1c', 2); // '-3pt, -2c'
     * ```
     *
     * @param input An instance or input which can be parsed into an instance.
     * @param operate A function to call when matching ranges are found and an
     *  operation should be performed between them. The range returned by this
     *  function ends up in the result.
     * @param operate.a The first range to operate on.
     * @param operate.b The second range to operate on.
     * @param remainder A function to call on a range that did not have a match
     *  in this instance where the range returned is added to the result.
     * @param remainder.a The remaining range to operate on.
     * @return A new instance.
     * @see [[Range.isExactMatch]]
     */
    Base.prototype.operate = function (input, operate, remainder) {
        var ranges = this.ranges;
        var output = [];
        var other = Parse.base(input);
        var otherRanges = other.ranges;
        var otherUsed = [];
        for (var i = 0; i < ranges.length; i++) {
            var range = ranges[i];
            for (var k = 0; k < otherRanges.length; k++) {
                if (!otherUsed[k]) {
                    var otherRange = otherRanges[k];
                    if (range.isExactMatch(otherRange)) {
                        range = operate(range, otherRange);
                        otherUsed[k] = true;
                    }
                }
            }
            output.push(range);
        }
        for (var k = 0; k < otherRanges.length; k++) {
            if (!otherUsed[k]) {
                output.push(remainder(otherRanges[k]));
            }
        }
        return new Base(this.input, output);
    };
    /**
     * Joins all ranges of the same classes together and then calculates all
     * equivalent ranges for each range for each valid group according to the
     * given options.
     *
     * *For example:*
     * ```javascript
     * uz('1.5pt').conversions(); // '3/16gal, 3/4qt, 1 1/2pt, 3c, 24floz, 48tbsp, 144tsp'
     * uz('20celsius, 45deg'); // '68F, 20celsius, 45deg, 0.785rad'
     * ```
     *
     * @param options Options to control which units and values are acceptable.
     * @return A new instance.
     * @see [[Transform]]
     * @see [[Core.globalTransform]]
     * @see [[Value.conversions]]
     */
    Base.prototype.conversions = function (options) {
        var transform = Core.globalTransform.extend(options);
        var compacted = this.compact(options);
        var ranges = compacted.ranges;
        var output = [];
        var _loop_2 = function (i) {
            var range = ranges[i];
            var convert = transform.convertWithMax ? range.max : range.min;
            convert.conversions(transform, false, function (transformed) {
                var min = transform.convertWithMax ? range.min.convertToValue(transformed.group, transformed.rateGroup) : transformed;
                var max = transform.convertWithMax ? transformed : range.max.convertToValue(transformed.group, transformed.rateGroup);
                if (min.value <= transform.max && max.value >= transform.min) {
                    output.push(new Range(min, max));
                }
            });
        };
        for (var i = 0; i < ranges.length; i++) {
            _loop_2(i);
        }
        return new Base(this.input, output);
    };
    /**
     * Executes the given function on each range in this instance and if the
     * function returns a valid range its added to the result.
     *
     * *For example:*
     * ```javascript
     * uz('1.5pt').mutate(r => r.scale(2)); // '3pt'
     * ```
     *
     * @param mutator The function which may return a range.
     * @return A new instance.
     */
    Base.prototype.mutate = function (mutator) {
        var ranges = [];
        var source = this.ranges;
        for (var i = 0; i < source.length; i++) {
            var mutated = mutator(source[i]);
            if (mutated && mutated.isValid) {
                ranges.push(mutated);
            }
        }
        return new Base(this.input, ranges);
    };
    /**
     * Removes the ranges from this instance that aren't valid according to the
     * transform options provided taking into account the global options.
     *
     * *For example:*
     * ```javascript
     * uz('1in, 2m').filter({system: Unitz.System.METRIC}); // '2m'
     * ```
     *
     * @param options Options to control which units and values are acceptable.
     * @return A new instance.
     * @see [[Transform]]
     * @see [[Core.globalTransform]]
     * @see [[Transform.isValidRange]]
     */
    Base.prototype.filter = function (options) {
        var transform = Core.globalTransform.extend(options);
        var ranges = this.ranges;
        var filtered = [];
        for (var i = 0; i < ranges.length; i++) {
            var range = ranges[i];
            if (transform.isValidRange(range)) {
                filtered.push(range);
            }
        }
        return new Base(this.input, filtered);
    };
    /**
     * Sorts the ranges in this instance based on the options provided taking into
     * account the global options.
     *
     * *For example:*
     * ```javascript
     * uz('1in, 3ft, 1.3yd, 1m').sort(); // 1.3yd, 1m, 3ft, 1in
     * uz('1in, 3ft, 1.3yd, 1m').sort({ascending: true}); // 1in, 3ft, 1m, 1.3yd
     * uz('1-3cups, 2-2.5cups, 4in').sort({
     *  type: Unitz.SortType.MIN,
     *  classes: {
     *   Volume: 1,
     *   Length: 2
     *  }
     * }); // 4in, 2 - 2.5cups, 1 - 3cups
     * ```
     *
     * @param options Options to control how sorting is done.
     * @return A new instance.
     * @see [[Sort]]
     * @see [[Core.globalSort]]
     */
    Base.prototype.sort = function (options) {
        var sort = Core.globalSort.extend(options);
        var ranges = this.ranges.slice();
        ranges.sort(sort.getSorter());
        return new Base(this.input, ranges);
    };
    /**
     * Calculates what this instance would need to be scaled by so that the given
     * value & unit pair is equal to the sum of ranges in this instance of the
     * same class. If there are no ranges with the same class then zero is
     * returned. If the sum of ranges with the same class results in an actual
     * range (where min != max) then you can specify how to pick a value from the
     * range with rangeDetla. A value of 0 uses the min, 1 uses the max, and 0.5
     * uses the average between them.
     *
     * *For example:*
     * ```javascript
     * uz('1m, 2 - 3c').getScaleTo('6c'); // 2
     * uz('1m, 2 - 3c').getScaleTo('6c', 0); // 3
     * uz('1m, 2 - 3c').getScaleTo('6c', 0.5); // 2.4
     * uz('1m, 2 - 3c').getScaleTo('45deg'); // 0
     * ```
  
     * @param unitValue A value & unit pair to scale the ranges in this instance to.
     * @param rangeDelta When this instance contains ranges this value instructs
     *  how the scale factor is calculated. A value of 0 means it looks at the
     *  minimum, 1 is the maximum, and 0.5 is the average.
     * @return A value to scale by or zero if this instance cannot match the input.
     * @see [[Base.convert]]
     * @see [[Parse.value]]
     */
    Base.prototype.getScaleTo = function (unitValue, rangeDelta) {
        if (rangeDelta === void 0) { rangeDelta = 1.0; }
        var to = Parse.value(unitValue);
        if (!to.isValid) {
            return 0;
        }
        var converted = this.convert(to.units());
        if (!converted || !converted.isValid) {
            return 0;
        }
        var convertedValue = (converted.maximum - converted.minimum) * rangeDelta + converted.minimum;
        var scale = to.value / convertedValue;
        return scale;
    };
    /**
     * Converts the ranges in this instance to a string with the given output
     * options taking into account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    Base.prototype.output = function (options) {
        var output = Core.globalOutput.extend(options);
        return output.ranges(this.ranges);
    };
    /**
     * Converts the appropriate ranges in this instance into the desired unit
     * and returns their converted sum. If the given unit does not map to a group
     * then null is returned. If there are no ranges in this instance in the same
     * class then the range returned is equivalent to zero.
     *
     * *For example:*
     * ```javascript
     * uz('1in, 1m, 1ft').convert('cm'); // '133.02 cm'
     * uz('60 mph').convert('miles per minute'); // '1 miles/minute'
     * ```
     *
     * @param unit The unit to calculate the sum of.
     * @return A new range which is the sum of ranges in the same class converted
     *  to the desired unit.
     * @see [[Core.getGroup]]
     * @see [[Range.isZero]]
     */
    Base.prototype.convert = function (unit) {
        var unitParsed = Parse.unit(unit);
        var group = Core.getGroup(unitParsed.unit);
        var rateGroup = Core.getGroup(unitParsed.rate);
        if (!group) {
            return null;
        }
        var ranges = this.ranges;
        var min = new Value(0, 0, 1, unit, group, unitParsed.rate, rateGroup);
        var max = new Value(0, 0, 1, unit, group, unitParsed.rate, rateGroup);
        for (var i = 0; i < ranges.length; i++) {
            var range = ranges[i];
            if (range.isMatch(min, max)) {
                min = min.add(range.min.convertToValue(group, rateGroup));
                max = max.add(range.max.convertToValue(group, rateGroup));
            }
        }
        return new Range(min, max);
    };
    /**
     * Alias for [[Base.convert]].
     */
    Base.prototype.to = function (unit) {
        return this.convert(unit);
    };
    /**
     * Iterates over each range in this instance in order or reversed and passes
     * each one to the given iterate function. If the iterate function returns
     * false the iteration will stop.
     *
     * @param iterate The function to invoke with each range and it's index.
     * @param iterate.range The current range being iterated.
     * @param iterate.index The index of the current range in this instance.
     * @param reverse Whether the iteration should be done forward or backward.
     * @return The reference to this instance.
     */
    Base.prototype.each = function (iterate, reverse) {
        if (reverse === void 0) { reverse = false; }
        var ranges = this.ranges;
        var start = reverse ? ranges.length - 1 : 0;
        var end = reverse ? -1 : ranges.length;
        var move = reverse ? -1 : 1;
        for (var i = start; i !== end; i += move) {
            if (iterate(ranges[i], i) === false) {
                break;
            }
        }
        return this;
    };
    /**
     * Returns an array of the classes represented in this instance. If there are
     * no classes in this instance then an empty array is returned.
     *
     * @return An array of the classes in this instance.
     */
    Base.prototype.classes = function () {
        var ranges = this.ranges;
        var classMap = {};
        var classes = [];
        for (var i = 0; i < ranges.length; i++) {
            var range = ranges[i];
            var group = range.min.group;
            if (group) {
                classMap[group.parent.name] = group.parent;
            }
        }
        for (var className in classMap) {
            classes.push(classMap[className]);
        }
        return classes;
    };
    Object.defineProperty(Base.prototype, "hasRanges", {
        /**
         * Returns whether this instance has actual ranges. An actual range is where
         * the minimum and maximum values differ.
         *
         * @see [[Range.isRange]]
         */
        get: function () {
            return this.test(false, false, function (r) { return r.isRange; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "hasRates", {
        /**
         * Returns whether this instance has values or ranges that are rates.
         *
         * @see [[Range.isRate]]
         */
        get: function () {
            return this.test(false, false, function (r) { return r.isRate; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "isValid", {
        /**
         * Returns whether this instance only has valid ranges. If any of the ranges
         * in this instance are not valid false is returned, otherwise true.
         *
         * @see [[Range.isValid]]
         */
        get: function () {
            return this.test(true, true, function (r) { return r.isValid; });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Performs a test on the ranges in this instance and returns whether the
     * ranges passed the test. If the `tester` function returns something
     * different than `expected` then the function ends early with `!passed`.
     * If all tests pass then `passed` is returned.
     *
     * @param expected The expected result of the tester.
     * @param passed The value to return if all ranges pass the test.
     * @param tester The function to test a range.
     * @return Return `passed` if all ranges return `expected` from `tester`.
     */
    Base.prototype.test = function (expected, passed, tester) {
        var ranges = this.ranges;
        for (var i = 0; i < ranges.length; i++) {
            if (tester(ranges[i]) != expected) {
                return !passed;
            }
        }
        return passed;
    };
    Object.defineProperty(Base.prototype, "length", {
        /**
         * Returns the number of ranges in this instance.
         */
        get: function () {
            return this.ranges.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "isFixed", {
        /**
         * Returns true if this instance has a single fixed value.
         *
         * @see [[Range.isFixed]]
         */
        get: function () {
            return this.ranges.length === 1 && this.ranges[0].isFixed;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "isRange", {
        /**
         * Returns true if this instance has a single range.
         *
         * @see [[Range.isRange]]
         */
        get: function () {
            return this.ranges.length === 1 && this.ranges[0].isRange;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "isRate", {
        /**
         * Returns true if this instance has a single rate.
         *
         * @see [[Range.isRate]]
         */
        get: function () {
            return this.ranges.length === 1 && this.ranges[0].isRate;
        },
        enumerable: true,
        configurable: true
    });
    return Base;
}());
export { Base };
//# sourceMappingURL=Base.js.map