"use strict";
import { Group } from './Group';
/**
 * A collection of groups and their units with the logic on how to convert
 * between groups with differing base units.
 *
 * A class is essentially something like "Length" where base units are "inches"
 * and "millimeters" and there are various other groups based off of these
 * base groups like "feet", "centimeters", and "meters".
 *
 * A class is responsible for being the sole place where conversion is done
 * between different groups in the same class.
 *
 * @see [[Class.convert]]
 */
var Class = (function () {
    /**
     * Creates a new instance of Class given the name of the class and optionally
     * the groups of the class.
     *
     * @param name The unique name of the class.
     * @param groups The optional list of groups to populate the class with.
     */
    function Class(name, groups) {
        this.name = name;
        this.groupMap = {};
        this.groups = [];
        this.converters = {};
        if (groups) {
            this.addGroups(groups);
        }
    }
    /**
     * Adds the group definitions to this class.
     *
     * @param definitions The array of group definitions.
     * @return The reference to this instance.
     * @see [[Class.addGroup]]
     */
    Class.prototype.addGroups = function (definitions) {
        for (var i = 0; i < definitions.length; i++) {
            this.addGroup(definitions[i]);
        }
        return this;
    };
    /**
     * Adds a group definition to this class. If the group is relative to another
     * group the [[Group.baseScale]] and [[Group.baseUnit]] are set to appropriate
     * values.
     *
     * @param definition The group definition.
     * @return The instance of the group created from the definition.
     * @see [[Class.addGroupUnit]]
     */
    Class.prototype.addGroup = function (definition) {
        var group = new Group(definition, this);
        var relativeUnit = group.relativeUnit, relativeScale = group.relativeScale, units = group.units;
        if (relativeUnit) {
            var relative = this.groupMap[relativeUnit];
            group.baseScale = relativeScale * relative.baseScale;
            group.baseUnit = relative.baseUnit;
        }
        for (var alias in units) {
            this.addGroupUnit(alias, group);
        }
        this.groups.push(group);
        return group;
    };
    /**
     * Adds the unit to this class for the given group. If the lowercase version
     * of the unit has not been mapped yet it will be mapped to the given group.
     *
     * @param unit The unit to map to the group.
     * @param group The group which has the unit.
     * @return The reference to this instance.
     */
    Class.prototype.addGroupUnit = function (unit, group) {
        var lower = unit.toLowerCase();
        this.groupMap[unit] = group;
        if (!this.groupMap[lower]) {
            this.groupMap[lower] = group;
        }
        return this;
    };
    /**
     * Removes the given unit associated to the given group from the class. If the
     * group is not mapped to this unit then this has no effect.
     *
     * @param unit The unit to remove from this class.
     * @param group The group which has the unit.
     * @return The reference to this instance.
     */
    Class.prototype.removeGroupUnit = function (unit, group) {
        var lower = unit.toLowerCase();
        if (this.groupMap[unit] === group) {
            delete this.groupMap[unit];
        }
        if (this.groupMap[lower] === group) {
            delete this.groupMap[lower];
        }
        return this;
    };
    /**
     * Determines the first group in this class which is a base group.
     *
     * @see [[Group.isBase]]
     */
    Class.prototype.getFirstBase = function () {
        var groups = this.groups;
        for (var i = 0; i < groups.length; i++) {
            var group = groups[i];
            if (group.isBase) {
                return group;
            }
        }
        return null;
    };
    /**
     * Updates the [[Group.classScale]] value in each group in this class so that
     * there is a baseline for comparing one group to another no matter the base
     * unit. For comparing in the same base, you can use [[Group.baseScale]].
     *
     * @return The reference to this instance.
     */
    Class.prototype.setClassScales = function () {
        var groups = this.groups;
        var first = this.getFirstBase();
        if (first) {
            for (var i = 0; i < groups.length; i++) {
                var group = groups[i];
                if (group.baseUnit === first.baseUnit) {
                    group.classScale = group.baseScale;
                }
                else if (group.baseUnit in this.converters) {
                    group.classScale = this.converters[group.baseUnit][first.baseUnit](group.baseScale);
                }
            }
        }
        return this;
    };
    /**
     * Sets the conversion function between the two base units.
     *
     * @param fromUnit The base unit to convert from.
     * @param toUnit The base unit to convert to.
     * @param converter The function to pass the value to convert.
     * @return The reference to this instance.
     */
    Class.prototype.setBaseConversion = function (fromUnit, toUnit, converter) {
        var converters = this.converters;
        converters[fromUnit] = converters[fromUnit] || {};
        converters[fromUnit][toUnit] = converter;
        return this;
    };
    /**
     * Determines which groups in this class are visible according to the given
     * transform. The groups can be iterated in reverse and can optionally take
     * a related group into consideration (when the system is GIVEN, we want to
     * return the groups with the same system).
     *
     * @param transform The transform which decides what groups are visible.
     * @param reverse If the groups of this class should be iterated in reverse.
     * @param relatedGroup A related group which may be used for visibility if the
     *  [[Transform.system]] is [[System.GIVEN]].
     * @param callback A function to invoke with all visible groups found and the
     *  index of that group in the set of visible groups. If `false` is returned
     *  by the function iteration of visible groups ceases.
     * @param callback.group The current visible group.
     * @param callback.index The index of the current visible group.
     * @see [[Transform.isVisibleGroup]]
     */
    Class.prototype.getVisibleGroups = function (transform, reverse, relatedGroup, callback) {
        var groups = this.groups;
        var matched = 0;
        var start = reverse ? groups.length - 1 : 0;
        var stop = reverse ? -1 : groups.length;
        var increment = reverse ? -1 : 1;
        for (var i = start; i !== stop; i += increment) {
            var group = groups[i];
            if (transform.isVisibleGroup(group, relatedGroup)) {
                var result = callback(group, matched++);
                if (result === false) {
                    break;
                }
            }
        }
    };
    /**
     * Converts the given number from a given group to a given group. If the two
     * groups are the same or one or both of the groups are not provided then the
     * `value` provided is returned. If the two groups have differing base units
     * the [[Class.converters]] map is used to convert the `value` over to the
     * proper base. If the [[Class.converters]] map is missing a base conversion
     * zero is returned. This might happen if a group is passed to this function
     * which does not belong to this class OR if the user has impromperly setup
     * their own classes.
     *
     * @param value The number to convert.
     * @param from The group of the number to convert from.
     * @param to The group to convert to.
     * @param invalid The value to return if a conversion between the two groups
     *  could not be made.
     * @return The converted number or zero if a base conversion could not be found.
     */
    Class.prototype.convert = function (value, from, to, invalid) {
        if (invalid === void 0) { invalid = 0; }
        if (from === to || !from || !to) {
            return value;
        }
        var converted = value * from.baseScale;
        if (from.baseUnit !== to.baseUnit) {
            var map = this.converters[from.baseUnit];
            if (!map || !map[to.baseUnit]) {
                return invalid;
            }
            var converter = map[to.baseUnit];
            converted = converter(converted);
        }
        return converted / to.baseScale;
    };
    return Class;
}());
export { Class };
//# sourceMappingURL=Class.js.map