"use strict";
import { Plurality } from './Plurality';
import { Functions as fn } from './Functions';
/**
 * A unit and its aliases as well as their plurality.
 *
 * A group is relative to a base group or is a base group itself. As unit
 * aliases are added to the group it determines the appropriate plural and
 * singular long and short versions given the unit aliases in this group.
 */
var Group = (function () {
    /**
     * Creates a new instance of Group given a definition and the parent class.
     *
     * @param definition The definition of the group.
     * @param parent The class which contains this group.
     */
    function Group(definition, parent) {
        /**
         * The scale of this group relative to the base group. This is used for
         * conversions of values with the same base group.
         */
        this.baseScale = 1;
        /**
         * The scale of this group relative to the first base group added to the
         * class. This is used to compare numbers of the same class across all bases.
         */
        this.classScale = 0;
        /**
         * Whether this group was dynamically created by user input having units
         * not mapped to groups by the developer.
         */
        this.dynamic = false;
        this.system = definition.system;
        this.common = !!definition.common;
        this.unit = definition.unit;
        this.baseUnit = definition.baseUnit;
        this.preferredUnit = definition.preferredUnit || definition.unit;
        this.relativeUnit = definition.relativeUnit;
        this.relativeScale = definition.relativeScale || 1;
        this.units = definition.units;
        this.denominators = definition.denominators;
        this.parent = parent;
        this.updateUnits();
    }
    Object.defineProperty(Group.prototype, "isBase", {
        /**
         * True if this group is a base group, otherwise false.
         */
        get: function () {
            return this.unit === this.baseUnit;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Sets the dynamic flag of this group.
     *
     * @param dynamic Whether this group is dynamic or not.
     * @return The reference to this instance.
     */
    Group.prototype.setDynamic = function (dynamic) {
        if (dynamic === void 0) { dynamic = true; }
        this.dynamic = dynamic;
        return this;
    };
    /**
     * Adds a denominator or array of denominators to this group.
     *
     * @param denominators A denominator or an array of denominators to add.
     * @return The reference to this instance.
     */
    Group.prototype.addDenominator = function (denominators) {
        fn.appendTo(this.denominators, denominators);
        return this;
    };
    /**
     * Sets the denominators of this group.
     *
     * @param denominators The new denominators for this group.
     * @return The reference to this instance.
     * @see [[Group.denominators]]
     */
    Group.prototype.setDenominators = function (denominators) {
        this.denominators = denominators;
        return this;
    };
    /**
     * Sets the common flag of this group.
     *
     * @param common Whether this group is common or not.
     * @return The reference to this instance.
     * @see [[Group.common]]
     */
    Group.prototype.setCommon = function (common) {
        if (common === void 0) { common = true; }
        this.common = common;
        return this;
    };
    /**
     * Sets the preferred unit of this group.
     *
     * @param unit The preferred unit of this group.
     * @return The reference to this instance.
     * @see [[Group.preferredUnit]]
     */
    Group.prototype.setPreferred = function (unit) {
        this.preferredUnit = unit;
        return this;
    };
    /**
     * Adds the given unit aliases to this group and the parent class.
     *
     * @param units The units to add to the group and class.
     * @return The reference to this instance.
     * @see [[Class.addGroupUnit]]
     */
    Group.prototype.addUnits = function (units) {
        var parent = this.parent;
        for (var unit in units) {
            this.units[unit] = units[unit];
            parent.addGroupUnit(unit, this);
        }
        this.updateUnits();
        return this;
    };
    /**
     * Removes the given unit aliases from this group and the parent class.
     *
     * @param units The array of unit aliases to remove.
     * @return The reference to this instance.
     * @see [[Class.removeGroupUnit]]
     */
    Group.prototype.removeUnits = function (units) {
        var parent = this.parent;
        var existing = this.units;
        for (var i = 0; i < units.length; i++) {
            var unit = units[i];
            if (unit in existing) {
                delete existing[unit];
                parent.removeGroupUnit(unit, this);
            }
        }
        return this;
    };
    /**
     * Updates the singular and plural long and short form units for this group.
     *
     * @return The reference to this instance.
     */
    Group.prototype.updateUnits = function () {
        this.singularShort = null;
        this.singularLong = null;
        this.pluralShort = null;
        this.pluralLong = null;
        for (var unit in this.units) {
            var plurality = this.units[unit];
            if (plurality !== Plurality.PLURAL) {
                if (!this.singularShort || unit.length < this.singularShort.length) {
                    this.singularShort = unit;
                }
                if (!this.singularLong || unit.length > this.singularLong.length) {
                    this.singularLong = unit;
                }
            }
            if (plurality !== Plurality.SINGULAR) {
                if (!this.pluralShort || unit.length < this.pluralShort.length) {
                    this.pluralShort = unit;
                }
                if (!this.pluralLong || unit.length > this.pluralLong.length) {
                    this.pluralLong = unit;
                }
            }
        }
        return this;
    };
    /**
     * Invokes a callback for each group in the parent class that are visible
     * based on the given transform relative to this group.
     *
     * @param transform The transform which decides what groups are visible.
     * @param reverse If the groups of the class should be iterated in reverse.
     * @param callback A function to invoke with all visible groups found and the
     *  index of that group in the set of visible groups. If `false` is returned
     *  by the function iteration of visible groups ceases.
     * @param callback.group The current visible group.
     * @param callback.index The index of the current visible group.
     * @see [[Transform.isVisibleGroup]]
     */
    Group.prototype.matches = function (transform, reverse, callback) {
        if (this.parent) {
            this.parent.getVisibleGroups(transform, reverse, this, callback);
        }
    };
    return Group;
}());
export { Group };
//# sourceMappingURL=Group.js.map