"use strict";
import { Functions as fn } from './Functions';
import { Core } from './Core';
import { Range } from './Range';
import { Value } from './Value';
import { Base } from './Base';
import { Translations } from './Translations';
import { Rates } from './Rates';
/**
 * The class which takes user input and parses it to specific structures.
 */
var Parse = (function () {
    function Parse() {
    }
    /**
     * Parses user input into a [[Base]] instance.
     *
     * @param input The input to parse into a Base.
     * @return The instance parsed from the input.
     */
    Parse.base = function (input) {
        if (input instanceof Base) {
            return input;
        }
        return new Base(input);
    };
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    Parse.ranges = function (input) {
        if (fn.isArray(input)) {
            return this.rangesFromArray(input);
        }
        else if (fn.isString(input)) {
            return this.rangesFromString(input);
        }
        else if (fn.isRangeDefinition(input)) {
            return this.rangesFromArray([input]);
        }
        else if (fn.isValueDefinition(input)) {
            return this.rangesFromArray([input]);
        }
        return [];
    };
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    Parse.rangesFromArray = function (input) {
        var ranges = [];
        for (var i = 0; i < input.length; i++) {
            var range = this.range(input[i]);
            ranges.push(range);
        }
        return ranges;
    };
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    Parse.rangesFromString = function (input) {
        var ranges = input.split(this.REGEX_LIST);
        return this.rangesFromArray(ranges);
    };
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    Parse.range = function (input) {
        if (fn.isString(input)) {
            return this.rangeFromString(input);
        }
        else if (fn.isRangeDefinition(input)) {
            var min = this.value(input.min);
            var max = this.value(input.max);
            return new Range(min, max);
        }
        else if (fn.isValueDefinition(input)) {
            var value = this.valueFromValue(input);
            return new Range(value, value);
        }
        return Range.INVALID;
    };
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    Parse.rangeFromString = function (input) {
        var matches = this.REGEX_RANGE.exec(input);
        if (!matches) {
            var fixed = this.valueFromString(input);
            return new Range(fixed, fixed);
        }
        var minInput = matches[1];
        var maxInput = matches[2];
        var minParsed = this.input(minInput);
        var maxParsed = this.input(maxInput);
        if (!minParsed || !maxParsed) {
            return Range.INVALID;
        }
        var minUnit = minParsed.unit || maxParsed.unit;
        var maxUnit = maxParsed.unit || minParsed.unit;
        var minRate = minParsed.rate || maxParsed.rate;
        var maxRate = maxParsed.rate || minParsed.rate;
        var min = this.valueFromResult(minParsed, minUnit, minRate);
        var max = this.valueFromResult(maxParsed, maxUnit, maxRate);
        return new Range(min, max);
    };
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    Parse.value = function (input) {
        if (fn.isString(input)) {
            return this.valueFromString(input);
        }
        else if (fn.isValueDefinition(input)) {
            return this.valueFromValue(input);
        }
        return Value.INVALID;
    };
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    Parse.valueFromValue = function (input) {
        var givenValue = fn.isDefined(input.value) ? input.value : 1;
        var num = fn.isDefined(input.num) ? input.num : givenValue;
        var den = fn.isDefined(input.den) ? input.den : 1;
        var parsedValue = fn.isDefined(input.value) ? input.value : num / den;
        var unit = input.unit || '';
        var rate = input.rate || '';
        var group = Core.getGroup(unit);
        var rateGroup = Core.getGroup(rate);
        return new Value(parsedValue, num, den, unit, group, rate, rateGroup);
    };
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    Parse.valueFromString = function (input) {
        var translated = Translations.translate(input);
        var parsed = this.input(translated);
        return parsed ? this.valueFromResult(parsed, parsed.unit, parsed.rate) : Value.INVALID;
    };
    /**
     * Parses user input into a [[Value]].
     *
     * @param result The already parsed input.
     * @param unit The unit parsed from the input.
     * @return The instance parsed from the input.
     */
    Parse.valueFromResult = function (result, unit, rateUnit) {
        var group = Core.getGroup(unit);
        var rateGroup = Core.getGroup(rateUnit);
        return new Value(result.value, result.valueNum, result.valueDen, unit, group, rateUnit, rateGroup);
    };
    /**
     * Parses user input into a [[ParseResult]]. If the input is not valid null
     * is returned.
     *
     * *Examples:*
     * - 1tsp
     * - 1 tsp
     * - 1/2 tsp
     * - 1 1/2 tsp
     * - -2 cups
     * - 2.35"
     *
     * @param input The string to parse a value and unit from.
     * @return The result of the parsing.
     */
    Parse.input = function (input) {
        var matches = this.REGEX_PARSE.exec(input);
        var whole = parseInt(matches[1]);
        var hasWhole = isFinite(whole);
        var sign = matches[1].charAt(0) === '-' ? -1 : 1;
        var num = parseInt(matches[3]);
        var den = parseInt(matches[5]);
        var decimal = matches[6];
        var hasDecimal = isFinite(parseFloat(decimal));
        var _a = this.unit(fn.trim(matches[7])), unit = _a.unit, rate = _a.rate;
        if (!hasWhole && hasDecimal) {
            whole = 0;
            hasWhole = true;
        }
        if (!hasWhole && !unit) {
            return null;
        }
        var value = 1;
        var valueDen = 1;
        var valueNum = 1;
        if (hasWhole) {
            value = whole;
            valueNum = whole;
            if (isFinite(den)) {
                valueDen = den;
                if (isFinite(num)) {
                    value += (num / den) * sign;
                    valueNum *= den;
                    valueNum += num;
                }
                else {
                    value /= den;
                }
            }
            else if (hasDecimal) {
                var remainder = parseFloat('0.' + decimal);
                value += remainder * sign;
                valueNum += remainder;
            }
            valueNum *= sign;
        }
        return { value: value, valueNum: valueNum, valueDen: valueDen, num: num, den: den, unit: unit, rate: rate };
    };
    /**
     * Parses unit input into a [[Rate]].
     *
     * *Examples:*
     * - m/s
     * - miles per hour
     * - mph
     *
     * @param input The string to parse a unit from.
     * @return The result of the parsing.
     */
    Parse.unit = function (input) {
        var rate = Rates.get(input);
        if (!rate) {
            var units = input.split(this.REGEX_UNIT);
            rate = {
                unit: units[0] ? fn.trim(units[0]).replace(/\.$/, '') : '',
                rate: units[2] ? fn.trim(units[2]).replace(/\.$/, '') : ''
            };
        }
        return rate;
    };
    /**
     * The regular expression used to split up a string into multiple ranges.
     */
    Parse.REGEX_LIST = /\s*,\s*/;
    /**
     * The regular expression used to split up a range string to determine the min
     * and maximum values.
     */
    Parse.REGEX_RANGE = /\s*(-?[^-]+)-(.+)/;
    /**
     * The regular expression used to parse a value number or fraction and
     * possible unit from a string.
     */
    Parse.REGEX_PARSE = /^\s*(-?\d*)(\s+(\d+))?(\s*\/\s*(\d+)|\.(\d+)|)\s*(.*)\s*$/i;
    /**
     * The regular expression used to split up a unit from a rateUnit.
     */
    Parse.REGEX_UNIT = /\s*(\/|\s+per\s+)\s*/i;
    return Parse;
}());
export { Parse };
//# sourceMappingURL=Parse.js.map