"use strict";
import { Functions as fn } from './Functions';
import { Value } from './Value';
import { Core } from './Core';
/**
 * A pair of minimum and maximum values. A range can be fixed which means the
 * minimum and maximum are equivalent - in which case the range behaves like
 * a [[Value]].
 */
var Range = (function () {
    /**
     * Creates a new instance of Range given the minimum and maximum values.
     *
     * @param min The minimum value for the range.
     * @param max The maximum value for the range.
     */
    function Range(min, max) {
        this.min = min.value < max.value ? min : max;
        this.max = max.value > min.value ? max : min;
    }
    Object.defineProperty(Range.prototype, "isValid", {
        /**
         * True if the min and max are both valid.
         */
        get: function () {
            return this.min.isValid && this.max.isValid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isFraction", {
        /**
         * True if the min or max are a fraction.
         */
        get: function () {
            return this.min.isFraction || this.max.isFraction;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isDecimal", {
        /**
         * True if the min and max are decimal.
         */
        get: function () {
            return this.min.isDecimal && this.max.isDecimal;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isRange", {
        /**
         * True if the min and max are not the same value.
         */
        get: function () {
            return this.min.value !== this.max.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isFixed", {
        /**
         * True if the min and max are the same value.
         */
        get: function () {
            return this.min.value === this.max.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isZero", {
        /**
         * True if the min and max are both equal to zero.
         */
        get: function () {
            return this.min.isZero && this.max.isZero;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isSingular", {
        /**
         * True if the min and max are both singular (1 or -1).
         */
        get: function () {
            return this.min.isSingular && this.max.isSingular;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "isRate", {
        /**
         * True if one of min and max are rates.
         */
        get: function () {
            return this.min.isRate && this.max.isRate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "average", {
        /**
         * The average number between the min and max.
         */
        get: function () {
            return (this.min.value + this.max.value) * 0.5;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "value", {
        /**
         * The minimum value of this range.
         */
        get: function () {
            return this.min.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "minimum", {
        /**
         * The minimum value of this range.
         */
        get: function () {
            return this.min.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "maximum", {
        /**
         * The maximum value of this range.
         */
        get: function () {
            return this.max.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Range.prototype, "unit", {
        /**
         * The unit which identifies the group of the minimum value or `null` if the
         * minimum value does not have a group.
         */
        get: function () {
            return this.min.group ? this.min.group.unit : null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Determines if the given range matches this range enough to allow a
     * simple mathematical operation between the two ranges.
     *
     * @param range The range to test.
     * @return True if the groups of the given range match this range.
     */
    Range.prototype.isExactMatch = function (range) {
        return this.min.group === range.min.group &&
            this.max.group === range.max.group &&
            this.min.rateGroup === range.min.rateGroup &&
            this.max.rateGroup === range.max.rateGroup;
    };
    /**
     * Determines if the given range matches this range enough to allow a
     * complex mathematical operation between the two ranges.
     *
     * @param min The minimum of the range to test.
     * @param max The maximum of the range to test.
     * @return True if the min and max have compatible values.
     */
    Range.prototype.isMatch = function (min, max) {
        return this.min.isMatch(min) && this.max.isMatch(max);
    };
    /**
     * @return A range which has the min and max converted to their preferred units.
     * @see [[Value.preferred]]
     */
    Range.prototype.preferred = function () {
        var min = this.min.preferred();
        var max = this.max.preferred();
        return new Range(min, max);
    };
    /**
     * @return A range which has only positive values. If the range is entirely
     *  negative then `null` is returned.
     */
    Range.prototype.positive = function () {
        var minNegative = this.min.value < 0;
        var maxNegative = this.max.value < 0;
        if (maxNegative) {
            return null;
        }
        var min = minNegative ? this.min.zero() : this.min.copy();
        var max = this.max.copy();
        return new Range(min, max);
    };
    /**
     * @return A range which has only negative values. If the range is entirely
     *  positive then `null` is returned.
     */
    Range.prototype.negative = function () {
        var minPositive = this.min.value >= 0;
        var maxPositive = this.max.value >= 0;
        if (minPositive) {
            return null;
        }
        var min = this.min.copy();
        var max = maxPositive ? this.max.zero() : this.max.copy();
        return new Range(min, max);
    };
    /**
     * @return A range which has a non-zero min and max. If both are equial to
     *  zero then `null` is returned.
     */
    Range.prototype.nonzero = function () {
        var minZero = fn.isZero(this.min.value);
        var maxZero = fn.isZero(this.max.value);
        if (minZero && maxZero) {
            return null;
        }
        var min = this.min.copy();
        var max = this.max.copy();
        return new Range(min, max);
    };
    /**
     * @return A range with only the maximum value from this range.
     */
    Range.prototype.maxd = function () {
        var fixed = this.max.copy();
        return new Range(fixed, fixed);
    };
    /**
     * @return A range with only the minimum value from this range.
     */
    Range.prototype.mind = function () {
        var fixed = this.min.copy();
        return new Range(fixed, fixed);
    };
    /**
     * Creates a range with with units that best represent the values. This may
     * cause the minimum and maximum values to have different units.
     *
     * @param transform Options to control which units and values are acceptable.
     * @param forOutput The output options that should be used to determine which
     *  value & unit is best.
     * @return A new range.
     * @see [[Value.normalize]]
     */
    Range.prototype.normalize = function (transform, forOutput) {
        var min = this.min.normalize(transform, forOutput);
        var max = this.max.normalize(transform, forOutput);
        return new Range(min, max);
    };
    /**
     * Adds this range and a given range (optionally scaled by a factor) together.
     *
     * @param addend The range to add to this instance.
     * @param scale The factor to multiply the addend by when added it to this
     *  instance.
     * @return a new range.
     * @see [[Value.add]]
     */
    Range.prototype.add = function (addend, scale) {
        if (scale === void 0) { scale = 1; }
        var min = this.min.add(addend.min, scale);
        var max = this.max.add(addend.max, scale);
        return new Range(min, max);
    };
    /**
     * Subtracts a given range (optionally scaled by a factor) from this range.
     *
     * @param subtrahend The range to remove from this instance.
     * @param scale The factor to multiply the subtrahend by when subtracting it
     *  from this instance.
     * @return A new range.
     * @see [[Value.sub]]
     */
    Range.prototype.sub = function (subtrahend, scale) {
        if (scale === void 0) { scale = 1; }
        var min = this.min.sub(subtrahend.min, scale);
        var max = this.max.sub(subtrahend.max, scale);
        return new Range(min, max);
    };
    /**
     * Multiplies this range by a scalar factor.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.scale]]
     */
    Range.prototype.scale = function (scale) {
        var min = this.min.scale(scale);
        var max = this.max.scale(scale);
        return new Range(min, max);
    };
    /**
     * Multiplies this range by a scale value.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.mul]]
     */
    Range.prototype.mul = function (scale) {
        var min = this.min.mul(scale);
        var max = this.max.mul(scale);
        return new Range(min, max);
    };
    /**
     * Returns a range which is coerced into being represented by fractions if a
     * valid fraction can be determined from the units valid denominators.
     *
     * @return A new range if the minimum and maximum are not fractions, otherwise
     *  the reference to this range is returned.
     * @see [[Value.fractioned]]
     */
    Range.prototype.fractioned = function () {
        if (this.min.isFraction && this.max.isFraction) {
            return this;
        }
        var min = this.min.fractioned();
        var max = this.max.fractioned();
        return new Range(min, max);
    };
    /**
     * Returns a range which has any fraction values converted to numbers.
     *
     * @return A new range if the mimimum or maximum are fractions, otherwise the
     *  the reference to this range is returned.
     * @see [[Value.numbered]]
     */
    Range.prototype.numbered = function () {
        if (!this.min.isFraction && !this.max.isFraction) {
            return this;
        }
        var min = this.min.numbered();
        var max = this.max.numbered();
        return new Range(min, max);
    };
    /**
     * Converts this range to a string with the given output options taking into
     * account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    Range.prototype.output = function (options) {
        var output = Core.globalOutput.extend(options);
        return output.range(this);
    };
    /**
     * Creates a fixed range from a given value. A fixed range behaves essentially
     * as a value since the minimum and maximum are equivalent.
     *
     * @param fixed The value to be used as the min and max of the range.
     * @return A new fixed range.
     */
    Range.fromFixed = function (fixed) {
        return new Range(fixed, fixed);
    };
    /**
     * A range instance which contains invalid values.
     */
    Range.INVALID = new Range(Value.INVALID, Value.INVALID);
    return Range;
}());
export { Range };
//# sourceMappingURL=Range.js.map