"use strict";
import { Core } from './Core';
import { Parse } from './Parse';
import { Value } from './Value';
/**
 * Creates a [[Translator]] which matches against a regular expression and when
 * the user input matches the regular expression another handler function is
 * called to translate the input. Optionally a constant value can be passed
 * to this function and down to the translator.
 *
 * @param regex The regular expression to match against user input.
 * @param handler The function to call if the input matched the expression.
 * @param vars The constant value to pass to the [[RegexTranslator]].
 * @return A [[Translator]] function.
 */
export function newRegexTranslator(regex, handler, vars) {
    return function (x) {
        var matches = x.match(regex);
        if (matches) {
            x = handler(matches, vars);
        }
        return x;
    };
}
/**
 * The class which holds [[Translator]]s to manipulate user input into something
 * more understandable to the [[Parse]] class.
 */
var Translations = (function () {
    function Translations() {
    }
    /**
     * Adds all translators in the library to be available when parsing.
     */
    Translations.addDefaults = function () {
        this.add(this.Quantity);
        this.add(this.NumberWords);
        this.add(this.FractionOfNumber);
        this.add(this.AndFraction);
        this.add(this.QuantityValue);
    };
    /**
     * Adds the given translator to the list of registered translators. This
     * translator will be called last.
     *
     * @param translator The function which translates user input.
  
     */
    Translations.add = function (translator) {
        this.registered.push(translator);
    };
    /**
     * Translates the user input based on the registered translators and returns
     * the final string ready to be parsed.
     *
     * @param input The input to translate.
     * @return The translated string.
     */
    Translations.translate = function (input) {
        var registered = this.registered;
        for (var i = 0; i < registered.length; i++) {
            input = registered[i](input);
        }
        return input;
    };
    /**
     * An array of translators that have been registered.
     *
     * @see [[Translations.add]]
     */
    Translations.registered = [];
    /**
     * A translator which takes a word which represents a number and converts it
     * the respective number.
     *
     * *Examples:*
     * - one [unit]
     * - dozen [unit]
     * - an eleven [unit]
     */
    Translations.NumberWords = newRegexTranslator(/^(an?\s+|)(one|two|three|four|five|six|seven|eight|nine|ten|eleven|twelve|dozen|thirteen|fourteen|fifteen|sixteen|seventeen|eighteen|nineteen|twenty|thirty|fourty|fifty|sixty|seventy|eighty|ninety)\s+(.*)/i, function (matches, vars) {
        var wordName = matches[2];
        var remaining = matches[3];
        return vars[wordName] + ' ' + remaining;
    }, {
        one: '1',
        two: '2',
        three: '3',
        four: '4',
        five: '5',
        six: '6',
        seven: '7',
        eight: '8',
        nine: '9',
        ten: '10',
        eleven: '11',
        twelve: '12',
        dozen: '12',
        thirteen: '13',
        fouteen: '14',
        fifteen: '15',
        sixteen: '16',
        seventeen: '17',
        eighteen: '18',
        nineteen: '19',
        twenty: '20',
        thirty: '30',
        fourty: '40',
        fifty: '50',
        sixty: '60',
        seventy: '70',
        eighty: '80',
        ninety: '90'
    });
    /**
     * A translator which takes a word which represents a fraction and multiplies
     * it by the following value.
     *
     * *Examples:*
     * - a third of an acre
     * - half a dozen eggs
     * - a seventh of a mile
     */
    Translations.FractionOfNumber = newRegexTranslator(/^(an?\s+|one|)(half|third|fourth|fifth|sixth|seventh|eighth|nineth|tenth)\s+(a\s+|an\s+|of\s+an?\s+|of\s+)(.*)/i, function (matches, vars) {
        var remaining = matches[4];
        var parsed = Parse.valueFromString(remaining);
        var fractionName = matches[2].toLowerCase();
        var fraction = vars[fractionName];
        return parsed.mul(fraction).output(Core.globalOutput);
    }, {
        half: Value.fromFraction(1, 2),
        third: Value.fromFraction(1, 3),
        fourth: Value.fromFraction(1, 4),
        fifth: Value.fromFraction(1, 5),
        sixth: Value.fromFraction(1, 6),
        seventh: Value.fromFraction(1, 7),
        eighth: Value.fromFraction(1, 8),
        nineth: Value.fromFraction(1, 9),
        tenth: Value.fromFraction(1, 10)
    });
    /**
     * A translator which takes a word which represents a fraction and multiplies
     * it by the following value.
     *
     * *Examples:*
     * - 23 and a half eggs
     * - one and a half acres
     * - 23 and a third
     * - 12 and one fourth
     */
    Translations.AndFraction = newRegexTranslator(/^(.*)\s+and\s+(an?|one)\s+(half|third|fourth|fifth|sixth|seventh|eighth|nineth|tenth)\s*(.*)/i, function (matches, vars) {
        var prefix = matches[1];
        var units = matches[4];
        var value = Parse.valueFromString(prefix + units);
        var fractionName = matches[3].toLowerCase();
        var fraction = vars[fractionName];
        return value.add(fraction).output(Core.globalOutput);
    }, {
        half: Value.fromFraction(1, 2),
        third: Value.fromFraction(1, 3),
        fourth: Value.fromFraction(1, 4),
        fifth: Value.fromFraction(1, 5),
        sixth: Value.fromFraction(1, 6),
        seventh: Value.fromFraction(1, 7),
        eighth: Value.fromFraction(1, 8),
        nineth: Value.fromFraction(1, 9),
        tenth: Value.fromFraction(1, 10)
    });
    /**
     * A translator which takes the amount in parenthesis and moves it out.
     *
     * *Examples:*
     * - (one and a half) acre
     * - (12) tacos
     */
    Translations.Quantity = newRegexTranslator(/^\((.*)\)(.*)$/, function (matches) {
        var quantity = matches[1];
        var unit = matches[2];
        return quantity + unit;
    });
    /**
     * A translator which takes the amount in parenthesis and moves it out.
     *
     * *Examples:*
     * - 1 (6 ounce)
     * - 5 (3 liter)
     */
    Translations.QuantityValue = newRegexTranslator(/^\s*((-?\d*)(\s+(\d+))?(\s*\/\s*(\d+)|\.(\d+)|))\s*\(\s*((-?\d*)(\s+(\d+))?(\s*\/\s*(\d+)|\.(\d+)|)\s*(.*))\s*\)\s*$/i, function (matches) {
        var quantityInput = matches[1];
        var quantity = Parse.valueFromString(quantityInput);
        var alternativeInput = matches[8];
        var alternative = Parse.valueFromString(alternativeInput);
        return alternative.mul(quantity).output(Core.globalOutput);
    });
    return Translations;
}());
export { Translations };
//# sourceMappingURL=Translations.js.map