"use strict";
import { Functions as fn } from './Functions';
import { Core } from './Core';
/**
 * A class which contains a parsed number or fraction.
 */
var Value = (function () {
    /**
     * Creates a new instance of Value given the value, possible numerator and
     * denominator, and the unit and it's group.
     *
     * @param value [[Value.value]]
     * @param num [[Value.num]]
     * @param den [[Value.den]]
     * @param unit [[Value.unit]]
     * @param group [[Value.group]]
     */
    function Value(value, num, den, unit, group, rate, rateGroup) {
        var divisor = fn.gcd(num, den);
        this.value = value;
        this.num = num / divisor;
        this.den = den / divisor;
        this.unit = unit;
        this.group = group;
        this.rate = rate;
        this.rateGroup = rateGroup;
    }
    Object.defineProperty(Value.prototype, "isValid", {
        /**
         * Returns true if this value was successfully parsed from some input.
         */
        get: function () {
            return isFinite(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isFraction", {
        /**
         * Returns true if this value is a fraction with a numerator and denoninator.
         */
        get: function () {
            return this.den !== 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isDecimal", {
        /**
         * Returns true if this value is a number and not a fraction.
         */
        get: function () {
            return this.den === 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isZero", {
        /**
         * Returns true if this value is zero.
         */
        get: function () {
            return fn.isZero(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isSingular", {
        /**
         * Returns true if this value is singular.
         *
         * @see [[Functions.isSingular]]
         */
        get: function () {
            return fn.isSingular(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "isRate", {
        /**
         * Returns true if this value is a rate.
         */
        get: function () {
            return !!this.rate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "scaled", {
        /**
         * Returns the number of this value relative to the base unit.
         */
        get: function () {
            return this.group ? this.value * this.group.baseScale : this.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "classScaled", {
        /**
         * Returns the number of this value relative to the first base unit of it's
         * class.
         */
        get: function () {
            return this.group ? this.value * this.group.classScale : this.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "calculated", {
        /**
         * Returns the number which represents the fraction in the value. There may
         * be a difference between this value and the number when the fraction is
         * calculated from the denominators of the group.
         */
        get: function () {
            return this.num / this.den;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "mixedWhole", {
        /**
         * Returns the whole number for the mixed fraction of this value. If this
         * value is not a fraction 0 is returned.
         */
        get: function () {
            return this.den !== 1 ? Math.floor(this.num / this.den) : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "mixedNum", {
        /**
         * Returns the numerator for the mixed fraction of this value. If this value
         * is not a fraction then the numerator is returned.
         */
        get: function () {
            return this.den !== 1 ? this.num % this.den : this.num;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "floor", {
        /**
         * Returns the floor of the number in this value.
         */
        get: function () {
            return Math.floor(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "ceil", {
        /**
         * Returns the ceiling of the number in this value.
         */
        get: function () {
            return Math.ceil(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "truncate", {
        /**
         * Returns the truncated number in this value taking into account it's sign.
         */
        get: function () {
            return this.value < 0 ? this.ceil : this.floor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "remainder", {
        /**
         * Returns the fractional part of the number in this value.
         */
        get: function () {
            return this.value - this.floor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "error", {
        /**
         * Returns the signed distance the number of this value is from the fraction
         * numerator and denominator determined. If this value is not a fraction then
         * this should return zero.
         */
        get: function () {
            return this.calculated - this.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "distance", {
        /**
         * Returns the absolute distance the number of this value is from the fraction
         * numerator and denominator determined. If this value is not a fraction then
         * this should return zero.
         */
        get: function () {
            return fn.abs(this.error);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Calculates the scale necessary to switch this value from the current rate
     * to the provided rate.
     *
     * @param rate The rate group.
     * @return The calculated scale.
     */
    Value.prototype.getRateScale = function (rate) {
        return rate ? rate.parent.convert(1, rate, this.rateGroup, 1) : 1;
    };
    /**
     * Determines whether the given value is compatible with this value to perform
     * operations with.
     *
     * @param other The value to test against.
     * @return True if the given value has compatible unit and rate groups.
     */
    Value.prototype.isMatch = function (other) {
        var group = this.group;
        var groupOther = other.group;
        var groupMatch = groupOther === group || (groupOther && group && groupOther.parent === group.parent);
        if (!groupMatch) {
            return false;
        }
        var rate = this.rateGroup;
        var rateOther = other.rateGroup;
        var rateMatch = rateOther === rate || (rateOther && rate && rateOther.parent === rate.parent);
        return rateMatch;
    };
    /**
     * Returns a version of this value with the preferred unit.
     *
     * @return A new value or the reference to this instance if it's groupless.
     * @see [[Group.preferredUnit]]
     */
    Value.prototype.preferred = function () {
        return this.group ? new Value(this.value, this.num, this.den, this.group.preferredUnit, this.group, this.rateGroup ? this.rateGroup.preferredUnit : this.rate, this.rateGroup) : this;
    };
    /**
     * Returns a copy of this value.
     *
     * @return A new value.
     */
    Value.prototype.copy = function () {
        return new Value(this.value, this.num, this.den, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Returns a value equivalent to zero with the unt and group of this instance.
     *
     * @return A new value.
     */
    Value.prototype.zero = function () {
        return new Value(0, 0, 1, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Returns the truncated version of this value. That's a value where the
     * number is a whole number.
     *
     * @return A new value.
     */
    Value.prototype.truncated = function () {
        return new Value(this.truncate, this.truncate, 1, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Returns a version of this value as a fraction.
     *
     * @return A new value or the reference to this instance if it's a fraction.
     */
    Value.prototype.fractioned = function () {
        if (this.isFraction) {
            return this;
        }
        if (this.group) {
            return Value.fromNumberWithDenominators(this.value, this.group.denominators, this.unit, this.group, this.rate, this.rateGroup);
        }
        return this;
    };
    /**
     * Returns a version of this value as a number.
     *
     * @return A new value or the reference to this instance if it's a number.
     */
    Value.prototype.numbered = function () {
        if (this.isFraction) {
            return new Value(this.value, this.value, 1, this.unit, this.group, this.rate, this.rateGroup);
        }
        return this;
    };
    /**
     * Converts this value to the given group and returns the result.
     *
     * @param to The group to convert to.
     * @param rate The group for the rate.
     * @return The converted value or the number of this value if there's no group.
     */
    Value.prototype.convertTo = function (to, rate) {
        if (rate === void 0) { rate = null; }
        var group = this.group;
        var rateScale = this.getRateScale(rate);
        var value = this.value * rateScale;
        return group ? group.parent.convert(value, group, to) : value;
    };
    /**
     * Converts this value to the given group and returns a new value. The new
     * value will attempted to be converted to a fraction.
     *
     * @param group The group to convert to.
     * @param rate The group for the rate.
     * @return A new value.
     */
    Value.prototype.convertToValue = function (group, rate) {
        if (rate === void 0) { rate = null; }
        return Value.fromNumberForGroup(this.convertTo(group, rate), group, rate || this.rateGroup);
    };
    /**
     * Determines the available conversions of this value for all groups
     * that are valid for the given transform.
     *
     * @param transform Transform which controls the units and values acceptable.
     * @param reverse Whether to iterate from largest units to smallest units
     *  (`true`), or from smallest to largest (`false`).
     * @param callback The function to invoke for each valid conversion.
     * @param callback.transformed The conversion calculated.
     * @param callback.index The index of the conversion during iteration.
     * @see [[Group.matches]]
     */
    Value.prototype.conversions = function (transform, reverse, callback) {
        var _this = this;
        var unitGroup = this.group;
        var rateGroup = this.rateGroup;
        var convertUnit = unitGroup && transform.convertUnit;
        var convertRate = rateGroup && transform.convertRate;
        var index = 0;
        if (convertUnit && convertRate) {
            rateGroup.matches(transform, reverse, function (rate) {
                unitGroup.matches(transform, reverse, function (group) {
                    callback(_this.convertToValue(group, rate), index++);
                });
            });
        }
        else if (convertUnit) {
            unitGroup.matches(transform, reverse, function (group) {
                callback(_this.convertToValue(group), index++);
            });
        }
        else if (convertRate) {
            rateGroup.matches(transform, reverse, function (rate) {
                callback(_this.convertToValue(unitGroup, rate), index++);
            });
        }
    };
    /**
     * Returns a value based on this value with the unit that best represents the
     * value. What is best is typically related to the magnitude of the value.
     * Really small and really large values are harder for people to comprehend so
     * the unit which results in the most normal looking value is determined.
     *
     * @param transform Transform which controls the units and values acceptable.
     * @param forOutput The output that may be used so the most normal looking
     *  value can be determined.
     * @return The most normal value found.
     * @see [[Value.conversions]]
     * @see [[Core.isMoreNormal]]
     */
    Value.prototype.normalize = function (transform, forOutput) {
        var closest;
        this.conversions(transform, false, function (convert) {
            var acceptable = !forOutput.isNumber(convert);
            if (!acceptable) {
                var number = forOutput.number(convert.value);
                acceptable = number !== '0';
            }
            if (acceptable) {
                if (!closest || Core.isMoreNormal(closest, convert, transform, forOutput)) {
                    closest = convert;
                }
            }
        });
        return closest || this;
    };
    /**
     * Calculates the sum of this value and the given addend scaled by some
     * factor. This is equivalent to `result = this + (addend * scale)`.
     *
     * @param addend The value to add to this.
     * @param scale The factor to scale the addend by before adding it to this.
     * @return A new instance.
     */
    Value.prototype.add = function (addend, scale) {
        if (scale === void 0) { scale = 1; }
        var rateScale = this.getRateScale(addend.rateGroup);
        var totalScale = rateScale * scale;
        var num = this.num * addend.den + addend.num * this.den * totalScale;
        var den = this.den * addend.den;
        var result = this.value + addend.value * totalScale;
        return new Value(result, num, den, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Calculates the difference between this value and the subtrahend scaled by
     * some factor. This is equivalent to `result = this - (subtrahend * scale)`.
     *
     * @param subtrahend The value to subtract from this.
     * @param scale The factor to scale the subtrahend by before subtraction.
     * @return A new instance.
     */
    Value.prototype.sub = function (subtrahend, scale) {
        if (scale === void 0) { scale = 1; }
        var rateScale = this.getRateScale(subtrahend.rateGroup);
        var totalScale = rateScale * scale;
        var num = this.num * subtrahend.den - subtrahend.num * this.den * totalScale;
        var den = this.den * subtrahend.den;
        var result = this.value - subtrahend.value * totalScale;
        return new Value(result, num, den, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Calculates a new value by multiplying this by a given factor. This is
     * equivalent to `result = this * scale`.
     *
     * @param scale The factor to scale this instance by.
     * @return A new instance.
     */
    Value.prototype.scale = function (scale) {
        return new Value(this.value * scale, this.num * scale, this.den, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Calculates a new value by multiplying this by a given value. This is
     * equivalent to `result = this * scale`.
     *
     * @param scale The value to scale this instance by.
     * @return A new instance.
     */
    Value.prototype.mul = function (scale) {
        var rateScale = this.getRateScale(scale.rateGroup);
        var num = this.num * scale.num * rateScale;
        var den = this.den * scale.den;
        var result = this.value * scale.value * rateScale;
        return new Value(result, num, den, this.unit, this.group, this.rate, this.rateGroup);
    };
    /**
     * Converts this value to a string with the given output options taking into
     * account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    Value.prototype.output = function (options) {
        var output = Core.globalOutput.extend(options);
        return output.value(this);
    };
    /**
     * Returns the units of this value as a string based on the global output
     * options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of the units of this value.
     * @see [[Output]]
     */
    Value.prototype.units = function (options) {
        var output = Core.globalOutput.extend(options);
        return output.units(this);
    };
    /**
     * Returns a Value instance which is a number with the optional unit and group.
     *
     * @param value The number.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    Value.fromNumber = function (value, unit, group, rate, rateGroup) {
        if (unit === void 0) { unit = ''; }
        if (group === void 0) { group = null; }
        if (rate === void 0) { rate = ''; }
        if (rateGroup === void 0) { rateGroup = null; }
        return new Value(value, value, 1, unit, group, rate, rateGroup);
    };
    /**
     * Returns a Value instance which tries to be a fraction given a range of
     * denominators. If the number is already whole or a fraction close
     * enough to the number cannot be found a value which is a number is returned.
     *
     * @param value The number to try to find a fraction for.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @param minDen The starting denominator to inclusively try.
     * @param maxDen The last denominator to inclusively try.
     * @return A new instance.
     */
    Value.fromNumberWithRange = function (value, unit, group, minDen, maxDen, rate, rateGroup) {
        if (unit === void 0) { unit = ''; }
        if (group === void 0) { group = null; }
        if (minDen === void 0) { minDen = 1; }
        if (maxDen === void 0) { maxDen = 100; }
        if (rate === void 0) { rate = ''; }
        if (rateGroup === void 0) { rateGroup = null; }
        var closestDenominator = 0;
        var closestDistance = -1;
        for (var i = minDen; i <= maxDen; i++) {
            var den = i;
            var num = Math.floor(den * value);
            var actual = num / den;
            var distance = fn.abs(value - actual);
            if (closestDistance === -1 || distance < closestDistance) {
                closestDistance = distance;
                closestDenominator = den;
            }
        }
        if (closestDistance > fn.EPSILON) {
            return new Value(value, value, 1, unit, group, rate, rateGroup);
        }
        if (closestDenominator === 0) {
            closestDenominator = 1;
        }
        return new Value(value, Math.floor(value * closestDenominator), closestDenominator, unit, group, rate, rateGroup);
    };
    /**
     * Returns a Value instance which tries to be a fraction based on the
     * denominators of the group. If a valid fraction could not be found then the
     * instance returned will be a number value. Since a unit is not passed here,
     * the preferred unit of the group is used as the unit of the value.
     *
     * @param value The number to try to find a fraction for.
     * @param group The group for the unit and also the denominators to try.
     * @param rateGroup The group for the rate.
     * @return A new instance.
     */
    Value.fromNumberForGroup = function (value, group, rateGroup) {
        return this.fromNumberWithDenominators(value, group.denominators, group.preferredUnit, group, rateGroup ? rateGroup.preferredUnit : '', rateGroup);
    };
    /**
     * Returns a Value instance which tries to be a fraction based on the
     * denominators of the group. If a valid fraction could not be found then the
     * instance returned will be a number value.
     *
     * @param value The number to try to find a fraction for.
     * @param denominators The array of denominators to try.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    Value.fromNumberWithDenominators = function (value, denominators, unit, group, rate, rateGroup) {
        if (unit === void 0) { unit = ''; }
        if (group === void 0) { group = null; }
        if (rate === void 0) { rate = ''; }
        if (rateGroup === void 0) { rateGroup = null; }
        var closestDenominator = 0;
        var closestDistance = -1;
        for (var i = 0; i < denominators.length; i++) {
            var den = denominators[i];
            var num = Math.floor(den * value);
            var actual = num / den;
            var distance = fn.abs(value - actual);
            if (closestDistance === -1 || distance < closestDistance) {
                closestDistance = distance;
                closestDenominator = den;
            }
        }
        if (closestDistance > fn.EPSILON) {
            return new Value(value, value, 1, unit, group, rate, rateGroup);
        }
        if (closestDenominator === 0) {
            closestDenominator = 1;
        }
        return new Value(value, Math.floor(value * closestDenominator), closestDenominator, unit, group, rate, rateGroup);
    };
    /**
     * Returns a Value instance for a given fraction specified by a numerator and
     * denominator.
     *
     * @param num The numerator of the fraction.
     * @param den The denominator of the fraction.
     * @param unit The unit, if any, of the fraction.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    Value.fromFraction = function (num, den, unit, group, rate, rateGroup) {
        if (unit === void 0) { unit = ''; }
        if (group === void 0) { group = null; }
        if (rate === void 0) { rate = ''; }
        if (rateGroup === void 0) { rateGroup = null; }
        return new Value(num / den, num, den, unit, group, rate, rateGroup);
    };
    /**
     * A value instance which contains invalid numbers.
     */
    Value.INVALID = new Value(Number.NaN, Number.NaN, 1, '', null, '', null);
    return Value;
}());
export { Value };
//# sourceMappingURL=Value.js.map