import { Class, ClassMap } from './Class';
import { Output } from './Output';
import { Group, GroupMap, GroupList } from './Group';
import { Transform } from './Transform';
import { Value } from './Value';
import { Sort } from './Sort';
/**
 * The global class which keeps track of all unit mappings and global options.
 *
 * This class is also responsible for creating dynamic classes and groups based
 * on approximation when a desired unit is not defined by the developer.
 */
export declare class Core {
    /**
     * The map of defined classes by their name.
     */
    static classMap: ClassMap;
    /**
     * An array of the defined classes.
     */
    static classes: Class[];
    /**
     * A map of groups by their acceptable units.
     */
    static unitToGroup: GroupMap;
    /**
     * A list of dynamically created groups based on units specified by a user
     * which are not defined by the developer.
     */
    static dynamicGroups: GroupList;
    /**
     * A map of the dynamically created groups by a key determined by
     * [[Core.getDynamicMatch]].
     */
    static dynamicMatches: GroupMap;
    /**
     * Dynamic groups are mapped together (by default) by looking at the first few
     * characters.
     *
     * @see [[Core.getDynamicMatch]]
     */
    static dynamicMatchLength: number;
    /**
     * The global options used for outputting [[Base]], [[Range]], and [[Value]]s
     * which may be overridden by specifying any number of options.
     *
     * @see [[Base.output]]
     * @see [[Range.output]]
     * @see [[Value.output]]
     */
    static globalOutput: Output;
    /**
     * The global transform options used for transforming a [[Base]] instance
     * by specifying what sort of units/groups are visible to the user.
     *
     * @see [[Base.normalize]]
     * @see [[Base.compact]]
     * @see [[Base.expand]]
     * @see [[Base.conversions]]
     * @see [[Base.filter]]
     */
    static globalTransform: Transform;
    /**
     * The global sort options used for ordering ranges in a [[Base]] instance.
     *
     * @see [[Base.sort]]
     */
    static globalSort: Sort;
    /**
     * Returns a [[Group]] instance mapped by the given unit. If no unit is given
     * `null` is returned. If the unit isn't mapped to a group a dynamic group
     * match is looked at and if none are found and `createDynamic` is true a new
     * dynamic group is created.
     *
     * @param unit The unit of the group to get.
     * @param createDynamic If creating a dynamic group should be created if an
     *  existing group could not be found.
     * @return The group matched to the unit or null if none was found.
     * @see [[Core.getDynamicMatch]]
     * @see [[Core.addDynamicUnit]]
     * @see [[Core.newDynamicGroup]]
     */
    static getGroup(unit: string, createDynamic?: boolean): Group;
    /**
     * Sets the given unit as the preferred unit for the group it belongs to. If a
     * group is not found then this has no affect.
     *
     * @param unit The unit to mark as the preferred unit.
     * @see [[Core.getGroup]]
     */
    static setPreferred(unit: string): void;
    /**
     * Sets whether the group associated with the given unit is common. A common
     * group is one a user is familiar with and would be okay seeing values
     * represented in. If a group is not found then this has no affect.
     *
     * @param unit The unit of a group to set the common flag.
     * @param common Whether the associated group should be common.
     * @see [[Core.getGroup]]
     */
    static setCommon(unit: string, common?: boolean): void;
    /**
     * Sets the denominators for the group associated to the given unit.
     * Denominators are useful for calculating a fraction from a value.
     *
     * @param unit The unit of a group to set the denominators of.
     * @param denominators The new denominators for the group.
     * @see [[Core.getGroup]]
     */
    static setDenominators(unit: string, denominators: number[]): void;
    /**
     * Adds the given class and all groups and units to the global state. If there
     * are units mapped to other groups they are overwritten by the units in the
     * given class.
     *
     * @param parent The class to add to the global state.
     */
    static addClass(parent: Class): void;
    /**
     * Adds an array of classes to the global state.
     *
     * @see [[Core.addClass]]
     */
    static addClasses(...classes: Class[]): void;
    /**
     * Adds the unit to the given dynamic group. This function also updates the
     * plurality of all the units currently in the group.
     *
     * @param unit The unit to add to the given group.
     * @param group The dynamically created group.
     * @return The instance of the given group.
     */
    static addDynamicUnit(unit: string, group: Group): Group;
    /**
     * Creates a dynamic class & group based on the given unit and adds it to the
     * global state. By default the group is marked with [[System.ANY]], is
     * common, and has the valid denominators 2, 3, 4, 5, 6, 8, 10.
     *
     * @param unit The initial unit of the group to use as the name of the class
     *  and the base unit of the group.
     * @return An instance of a new Group with a new parent Class.
     */
    static newDynamicGroup(unit: string): Group;
    /**
     * The function which takes a unit and generates a string which should be used
     * to mark similarly spelled units under the same dynamic group.
     *
     * @param unit The unit to build a key from.
     * @return The key which identifies the dynamic group.
     */
    static getDynamicMatch(unit: string): string;
    /**
     * The function which takes to values and determines which one is more
     * "normal" or "human friendly".
     *
     * @param fromValue The most normal value found so far.
     * @param toValue The value to compare to.
     * @param transform The transformation rules to guide the function to choose
     *  the more normal value.
     * @param forOutput The output options to guide the function to choose the
     *  more normal value.
     * @return True if `toValue` appears more normal than `fromValue`.
     */
    static isMoreNormal(fromValue: Value, toValue: Value, transform: Transform, forOutput: Output): boolean;
}
