import { GroupDefinition, RangeDefinition, ValueDefinition } from './Types';
/**
 * The class which contains commonly used functions by the library. These
 * functions and variables exist in a class so they may be overridden if
 * desired.
 */
export declare class Functions {
    /**
     * The maximum distance a number can be from another to be considered
     * equivalent. This is to compensate for floating point precision issues.
     */
    static EPSILON: number;
    /**
     * Determines if the given number is zero.
     *
     * @param x The number to test.
     * @return True if the number is zero, otherwise false.
     * @see [[Functions.EPSILON]]
     */
    static isZero(x: number): boolean;
    /**
     * Determines if the given number is equal to another.
     *
     * @param a The first number to compare.
     * @param b The second number to compare.
     * @return True if the two numbers are equal.
     * @see [[Functions.EPSILON]]
     */
    static isEqual(a: number, b: number): boolean;
    /**
     * Determines if the given number is a whole number (integer).
     *
     * @param x The number to test.
     * @return True if the number is whole, otherwise false.
     * @see [[Functions.EPSILON]]
     */
    static isWhole(x: number): boolean;
    /**
     * Determines if the given number is singular. A singular number is 1 or -1.
     *
     * @param x The number to test.
     * @return True if the number is singular, otherwise false.
     * @see [[Functions.EPSILON]]
     */
    static isSingular(x: any): boolean;
    /**
     * Determines if the given number is valid. A valid number is finite and not
     * NaN or Infinity.
     *
     * @param x The number to test.
     * @return True if the input is finite number.
     */
    static isNumber(x: any): boolean;
    /**
     * Trims the given input if its a string.
     *
     * @param x The string to remove space from the beginning and end.
     * @return A trimmed string.
     */
    static trim(x: string): string;
    /**
     * Calculates the greatest common denominator between the two numbers. If
     * either of the numbers are not whole (integers) then 1 is immediately
     * returned.
     *
     * @param a The first number.
     * @param b The second number.
     * @return The greatest common denominator between the two numbers.
     */
    static gcd(a: number, b: number): number;
    /**
     * Determines the absolute value of the given number.
     *
     * @param x The number to return the positive version of.
     * @return The absolute value of x.
     */
    static abs(x: number): number;
    /**
     * Determines the sign of the given number. One of three values will be
     * returned: 1, 0, or -1.
     *
     * @param x The number to determine the sign of.
     * @return The sign of the given number.
     */
    static sign(x: number): number;
    /**
     * Appends an element or array of elements to the end of the given array.
     *
     * @param array The array to append values to the end of.
     * @param input The element or array of elements to append to the end.
     * @return The reference to the `array` given.
     */
    static appendTo<T>(array: T[], input: T | T[]): T[];
    /**
     * Determines whether the given input looks like a [[GroupDefinition]].
     *
     * @param input The variable to inspect.
     * @return True if the variable appears to be a [[GroupDefinition]].
     */
    static isGroupDefinition(input: any): input is GroupDefinition;
    /**
     * Determines whether the given input looks like a [[ValueDefinition]].
     *
     * @param input The variable to inspect.
     * @return True if the variable appears to be a [[ValueDefinition]].
     */
    static isValueDefinition(input: any): input is ValueDefinition;
    /**
     * Determines whether the given input looks like a [[RangeDefinition]].
     *
     * @param input The variable to inspect.
     * @return True if the variable appears to be a [[RangeDefinition]].
     */
    static isRangeDefinition(input: any): input is RangeDefinition;
    /**
     * Determines whether the given input is an array.
     *
     * @param input The variable to test.
     * @return True if the variable is an array, otherwise false.
     */
    static isArray<T>(input: any): input is Array<T>;
    /**
     * Determines whether the given input is a string.
     *
     * @param input The variable to test.
     * @return True if the variable is a string, otherwise false.
     */
    static isString(input: any): input is string;
    /**
     * Determines whether the given input is defined.
     *
     * @param input The variable to test.
     * @return True if the variable is defined, otherwise false.
     */
    static isDefined(input: any): boolean;
    /**
     * Returns the first argument which is defined.
     *
     * @param a The first argument to look at.
     * @param b The second argument to look at.
     * @return The first defined argument.
     * @see [[Functions.isDefined]]
     */
    static coalesce(a: any, b: any): any;
}
