import { System } from './System';
import { Class } from './Class';
import { GroupDefinition, UnitDefinitionMap, Numbers } from './Types';
import { Transform } from './Transform';
/**
 * An array of groups.
 */
export declare type GroupList = Group[];
/**
 * An object of groups keyed by their units.
 */
export declare type GroupMap = {
    [key: string]: Group;
};
/**
 * A function which takes a unit and returns a Group instance.
 */
export declare type GroupFactory = (unit: string) => Group;
/**
 * A unit and its aliases as well as their plurality.
 *
 * A group is relative to a base group or is a base group itself. As unit
 * aliases are added to the group it determines the appropriate plural and
 * singular long and short versions given the unit aliases in this group.
 */
export declare class Group {
    /**
     * The System this group belongs in. One of [[System.METRIC]],
     * [[System.US]], or [[System.ANY]].
     */
    system: System;
    /**
     * Whether this group is considered common. A common group is one a
     * developer has decided the user will be familiar with and would be okay
     * seeing values in this unit.
     */
    common: boolean;
    /**
     * The main unit for this group. This is the preferred unit if one is not
     * specified in the definition.
     */
    unit: string;
    /**
     * The unit of the base group. The base group is typically the smallest value
     * and is also where the baseUnit matches unit.
     */
    baseUnit: string;
    /**
     * The scale of this group relative to the base group. This is used for
     * conversions of values with the same base group.
     */
    baseScale: number;
    /**
     * The scale of this group relative to the first base group added to the
     * class. This is used to compare numbers of the same class across all bases.
     */
    classScale: number;
    /**
     * The unit the developer prefers for their users.
     *
     * @see [[Base.preferred]]
     */
    preferredUnit: string;
    /**
     * The unit this group is relative to.
     */
    relativeUnit: string;
    /**
     * This defines how to calculate the scale of the group by multiplying it by
     * the group of `relativeUnit`.
     */
    relativeScale: number;
    /**
     * A map of all valid units and whether they are singular, plural, or either.
     */
    units: UnitDefinitionMap;
    /**
     * The list of valid denominators for this group. Values when converted to
     * fractions look at this list to ensure that fractions are not produced that
     * are not user friendly. For example, no one would say 5/23 a meter.
     */
    denominators: number[];
    /**
     * Whether this group was dynamically created by user input having units
     * not mapped to groups by the developer.
     */
    dynamic: boolean;
    /**
     * The class this group belongs to.
     */
    parent: Class;
    /**
     * The shortest unit in this group that is used for singular values.
     */
    singularShort: string;
    /**
     * The longest unit in this group that is used for singular values.
     */
    singularLong: string;
    /**
     * The longest unit in this group that is used for plural values.
     */
    pluralShort: string;
    /**
     * The longest unit in this group that is used for plural values.
     */
    pluralLong: string;
    /**
     * Creates a new instance of Group given a definition and the parent class.
     *
     * @param definition The definition of the group.
     * @param parent The class which contains this group.
     */
    constructor(definition: GroupDefinition, parent: Class);
    /**
     * True if this group is a base group, otherwise false.
     */
    readonly isBase: boolean;
    /**
     * Sets the dynamic flag of this group.
     *
     * @param dynamic Whether this group is dynamic or not.
     * @return The reference to this instance.
     */
    setDynamic(dynamic?: boolean): this;
    /**
     * Adds a denominator or array of denominators to this group.
     *
     * @param denominators A denominator or an array of denominators to add.
     * @return The reference to this instance.
     */
    addDenominator(denominators: Numbers): this;
    /**
     * Sets the denominators of this group.
     *
     * @param denominators The new denominators for this group.
     * @return The reference to this instance.
     * @see [[Group.denominators]]
     */
    setDenominators(denominators: number[]): this;
    /**
     * Sets the common flag of this group.
     *
     * @param common Whether this group is common or not.
     * @return The reference to this instance.
     * @see [[Group.common]]
     */
    setCommon(common?: boolean): this;
    /**
     * Sets the preferred unit of this group.
     *
     * @param unit The preferred unit of this group.
     * @return The reference to this instance.
     * @see [[Group.preferredUnit]]
     */
    setPreferred(unit: string): this;
    /**
     * Adds the given unit aliases to this group and the parent class.
     *
     * @param units The units to add to the group and class.
     * @return The reference to this instance.
     * @see [[Class.addGroupUnit]]
     */
    addUnits(units: UnitDefinitionMap): this;
    /**
     * Removes the given unit aliases from this group and the parent class.
     *
     * @param units The array of unit aliases to remove.
     * @return The reference to this instance.
     * @see [[Class.removeGroupUnit]]
     */
    removeUnits(units: string[]): this;
    /**
     * Updates the singular and plural long and short form units for this group.
     *
     * @return The reference to this instance.
     */
    updateUnits(): this;
    /**
     * Invokes a callback for each group in the parent class that are visible
     * based on the given transform relative to this group.
     *
     * @param transform The transform which decides what groups are visible.
     * @param reverse If the groups of the class should be iterated in reverse.
     * @param callback A function to invoke with all visible groups found and the
     *  index of that group in the set of visible groups. If `false` is returned
     *  by the function iteration of visible groups ceases.
     * @param callback.group The current visible group.
     * @param callback.index The index of the current visible group.
     * @see [[Transform.isVisibleGroup]]
     */
    matches(transform: Transform, reverse: boolean, callback: (group: Group, index: number) => any): void;
}
