import { Value } from './Value';
import { Group } from './Group';
import { Range, RangeList } from './Range';
/**
 * The enumeration which decides what unit to use when converting to a string.
 */
export declare enum OutputUnit {
    /**
     * This value will keep units from being displayed.
     */
    NONE = 0,
    /**
     * This value will ensure the unit exactly as the user entered it is used in
     * the output no matter whether the value's plurality matches the given
     * unit's plurality.
     *
     * @see [[Value.unit]]
     */
    GIVEN = 1,
    /**
     * This value will force the short versions of the unit to be used.
     *
     * @see [[Group.singularShort]]
     * @see [[Group.pluralShort]]
     */
    SHORT = 2,
    /**
     * This value will force the long versions of the unit to be used.
     *
     * @see [[Group.singularLong]]
     * @see [[Group.pluralLong]]
     */
    LONG = 3,
}
/**
 * The enumeration which decides how a value will be converted to a string.
 */
export declare enum OutputFormat {
    /**
     * The format of the user input will be used if possible.
     */
    GIVEN = 0,
    /**
     * All values will be displayed using their decimal representation.
     */
    NUMBER = 1,
    /**
     * All values will be displayed as a mixed fraction if the value is a fraction.
     * A mixed fraction has a whole number followed by a fraction where the
     * numerator is smaller than the denominator.
     *
     * @see [[Value.isFraction]]
     */
    MIXED = 2,
    /**
     * All values will be displayed as an improper fraction if the value is a
     * fraction and the numerator is larger than the denoninator.
     *
     * @see [[Value.isFraction]]
     */
    IMPROPER = 3,
}
/**
 * Developer input which can be passed to an Output constructor or all the
 * various functions that use the [[Output]] object.
 *
 * @see [[Base.output]]
 * @see [[Range.output]]
 * @see [[Value.output]]
 */
export interface OutputInput {
    /**
     * @see [[Output.unit]]
     */
    unit?: OutputUnit;
    /**
     * @see [[Output.format]]
     */
    format?: OutputFormat;
    /**
     * @see [[Output.repeatUnit]]
     */
    repeatUnit?: boolean;
    /**
     * @see [[Output.unitSpacer]]
     */
    unitSpacer?: string;
    /**
     * @see [[Output.rangeSpacer]]
     */
    rangeSpacer?: string;
    /**
     * @see [[Output.fractionSpacer]]
     */
    fractionSpacer?: string;
    /**
     * @see [[Output.mixedSpacer]]
     */
    mixedSpacer?: string;
    /**
     * @see [[Output.rateSpacer]]
     */
    rateSpacer?: string;
    /**
     * @see [[Output.delimiter]]
     */
    delimiter?: string;
    /**
     * @see [[Output.significant]]
     */
    significant?: number;
}
/**
 * The class which converts Unitz objects to strings.
 */
export declare class Output implements OutputInput {
    /**
     * The option that specifies which units are chosen.
     */
    unit: OutputUnit;
    /**
     * The option that specifies how values are displayed.
     */
    format: OutputFormat;
    /**
     * Whether or not a unit should be displayed for the minimum and maximum of a
     * range when they have the same group.
     */
    repeatUnit: boolean;
    /**
     * The spacing used between the value and the unit.
     */
    unitSpacer: string;
    /**
     * The spacing used between the minimum and maximum values in a range.
     */
    rangeSpacer: string;
    /**
     * The spacing used to separate the numerator and denominator of a fraction.
     */
    fractionSpacer: string;
    /**
     * The spacing used to seperate a mixed number from the fraction.
     */
    mixedSpacer: string;
    /**
     * The spacing used to separate a unit and the rate unit.
     */
    rateSpacer: string;
    /**
     * The delimiter used to separate ranges.
     */
    delimiter: string;
    /**
     * An option used to restrict numbers from displaying large decimal numbers.
     * When this value is set to -1 numbers are displayed fully. If the value is
     * set to zero all numbers will be truncated to the whole version.
     */
    significant: number;
    /**
     * Creates a new instance of Output with an optional set of options to
     * override the default values.
     *
     * @param input The options to apply to the new instance.
     */
    constructor(input?: OutputInput);
    /**
     * Overrides values in this instance with ones specified in input.
     *
     * @param input The values to override.
     * @return The reference to this instance.
     */
    set(input: OutputInput): this;
    /**
     * Returns an Output instance which matches the desired options. If no options
     * are specified the reference to this instance is returned. If the options
     * are already an instance of Output its returned. If options are specified
     * a new instance is created with the options of this instance, and the given
     * options applied with [[Output.set]].
     *
     * @param input The options desired.
     * @return An instance of this class which matches the desired options.
     */
    extend(input?: OutputInput): Output;
    /**
     * Converts the list of ranges to a string. If a range is not valid it is
     * skipped.
     *
     * @param ranges The list of ranges to convert.
     * @return The string representation of the input.
     */
    ranges(ranges: RangeList): string;
    /**
     * Converts the range to a string.
     *
     * @param ranges The range to convert.
     * @return The string representation of the input.
     */
    range(range: Range): string;
    /**
     * Converts the value to the string optionally showing or hiding the unit.
     *
     * @param value The value to convert.
     * @param showUnit Whether or not the unit should be added to the string.
     * @return The string representation of the input.
     */
    value(value: Value, showUnit?: boolean): string;
    /**
     * Generates a full unit string including the rate unit if it exists.
     *
     * @param value The value to generate a unit for.
     * @return The units string representation.
     */
    units(value: Value): string;
    /**
     * Generates a unit string given the value, the current unit, and its group.
     *
     * @param value The value to generate a unit for.
     * @param unit The unit to potentially use.
     * @param group The group of the unit.
     * @return The unit determined based on the options.
     */
    group(value: number, unit: string, group: Group): string;
    /**
     * Converts the number to a string.
     *
     * @param x The number to convert.
     * @return The string representation of the input.
     */
    number(x: number): string;
    /**
     * Determines whether the value should be displayed as a fraction.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a fraction, otherwise false.
     */
    isFraction(value: Value): boolean;
    /**
     * Determines whether the value should be displayed as a number.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a number, otherwise false.
     */
    isNumber(value: Value): boolean;
    /**
     * Determines whether the value should be displayed as a mixed fraction. This
     * assumes [[Output.isFraction]] was already checked and returned true.
     *
     * @param value The value to look at.
     * @return True if the value should be displayed as a mixed fraction, otherwise false.
     */
    isMixed(value: Value): boolean;
    /**
     * Determines whether the short unit should be displayed.
     *
     * @param group The group of the unit.
     * @return True if the short unit should be displayed, otherwise false.
     */
    isShortUnit(group: Group): boolean;
    /**
     * Determines whether the long unit should be displayed.
     *
     * @param group The group of the unit.
     * @return True if the short unit should be displayed, otherwise false.
     */
    isLongUnit(group: Group): boolean;
}
