import { BaseInput, RangeInput, RangesInput, ValueInput, ValueDefinition, ParseResult } from './Types';
import { Range, RangeList } from './Range';
import { Value } from './Value';
import { Base } from './Base';
import { Rate } from './Rates';
/**
 * The class which takes user input and parses it to specific structures.
 */
export declare class Parse {
    /**
     * The regular expression used to split up a string into multiple ranges.
     */
    static REGEX_LIST: RegExp;
    /**
     * The regular expression used to split up a range string to determine the min
     * and maximum values.
     */
    static REGEX_RANGE: RegExp;
    /**
     * The regular expression used to parse a value number or fraction and
     * possible unit from a string.
     */
    static REGEX_PARSE: RegExp;
    /**
     * The regular expression used to split up a unit from a rateUnit.
     */
    static REGEX_UNIT: RegExp;
    /**
     * Parses user input into a [[Base]] instance.
     *
     * @param input The input to parse into a Base.
     * @return The instance parsed from the input.
     */
    static base(input: BaseInput): Base;
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static ranges(input: RangesInput): RangeList;
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static rangesFromArray(input: RangeInput[]): RangeList;
    /**
     * Parses user input into a an array of [[Range]]s.
     *
     * @param input The input to parse.
     * @return The instances parsed from the input.
     */
    static rangesFromString(input: string): RangeList;
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static range(input: RangeInput): Range;
    /**
     * Parses user input into a [[Range]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static rangeFromString(input: string): Range;
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static value(input: ValueInput): Value;
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static valueFromValue(input: ValueDefinition): Value;
    /**
     * Parses user input into a [[Value]].
     *
     * @param input The input to parse.
     * @return The instance parsed from the input.
     */
    static valueFromString(input: string): Value;
    /**
     * Parses user input into a [[Value]].
     *
     * @param result The already parsed input.
     * @param unit The unit parsed from the input.
     * @return The instance parsed from the input.
     */
    static valueFromResult(result: ParseResult, unit: string, rateUnit: string): Value;
    /**
     * Parses user input into a [[ParseResult]]. If the input is not valid null
     * is returned.
     *
     * *Examples:*
     * - 1tsp
     * - 1 tsp
     * - 1/2 tsp
     * - 1 1/2 tsp
     * - -2 cups
     * - 2.35"
     *
     * @param input The string to parse a value and unit from.
     * @return The result of the parsing.
     */
    static input(input: string): ParseResult;
    /**
     * Parses unit input into a [[Rate]].
     *
     * *Examples:*
     * - m/s
     * - miles per hour
     * - mph
     *
     * @param input The string to parse a unit from.
     * @return The result of the parsing.
     */
    static unit(input: string): Rate;
}
