import { Value } from './Value';
import { Transform } from './Transform';
import { Output, OutputInput } from './Output';
/**
 * An array of ranges.
 */
export declare type RangeList = Range[];
/**
 * A function which takes a range and might return a range based on the given.
 */
export declare type RangeMutator = (range: Range) => Range;
/**
 * A pair of minimum and maximum values. A range can be fixed which means the
 * minimum and maximum are equivalent - in which case the range behaves like
 * a [[Value]].
 */
export declare class Range {
    /**
     * A range instance which contains invalid values.
     */
    static INVALID: Range;
    /**
     * The minimum value in the range.
     */
    readonly min: Value;
    /**
     * The maximum value in the range.
     */
    readonly max: Value;
    /**
     * Creates a new instance of Range given the minimum and maximum values.
     *
     * @param min The minimum value for the range.
     * @param max The maximum value for the range.
     */
    constructor(min: Value, max: Value);
    /**
     * True if the min and max are both valid.
     */
    readonly isValid: boolean;
    /**
     * True if the min or max are a fraction.
     */
    readonly isFraction: boolean;
    /**
     * True if the min and max are decimal.
     */
    readonly isDecimal: boolean;
    /**
     * True if the min and max are not the same value.
     */
    readonly isRange: boolean;
    /**
     * True if the min and max are the same value.
     */
    readonly isFixed: boolean;
    /**
     * True if the min and max are both equal to zero.
     */
    readonly isZero: boolean;
    /**
     * True if the min and max are both singular (1 or -1).
     */
    readonly isSingular: boolean;
    /**
     * True if one of min and max are rates.
     */
    readonly isRate: boolean;
    /**
     * The average number between the min and max.
     */
    readonly average: number;
    /**
     * The minimum value of this range.
     */
    readonly value: number;
    /**
     * The minimum value of this range.
     */
    readonly minimum: number;
    /**
     * The maximum value of this range.
     */
    readonly maximum: number;
    /**
     * The unit which identifies the group of the minimum value or `null` if the
     * minimum value does not have a group.
     */
    readonly unit: string;
    /**
     * Determines if the given range matches this range enough to allow a
     * simple mathematical operation between the two ranges.
     *
     * @param range The range to test.
     * @return True if the groups of the given range match this range.
     */
    isExactMatch(range: Range): boolean;
    /**
     * Determines if the given range matches this range enough to allow a
     * complex mathematical operation between the two ranges.
     *
     * @param min The minimum of the range to test.
     * @param max The maximum of the range to test.
     * @return True if the min and max have compatible values.
     */
    isMatch(min: Value, max: Value): boolean;
    /**
     * @return A range which has the min and max converted to their preferred units.
     * @see [[Value.preferred]]
     */
    preferred(): Range;
    /**
     * @return A range which has only positive values. If the range is entirely
     *  negative then `null` is returned.
     */
    positive(): Range;
    /**
     * @return A range which has only negative values. If the range is entirely
     *  positive then `null` is returned.
     */
    negative(): Range;
    /**
     * @return A range which has a non-zero min and max. If both are equial to
     *  zero then `null` is returned.
     */
    nonzero(): Range;
    /**
     * @return A range with only the maximum value from this range.
     */
    maxd(): Range;
    /**
     * @return A range with only the minimum value from this range.
     */
    mind(): Range;
    /**
     * Creates a range with with units that best represent the values. This may
     * cause the minimum and maximum values to have different units.
     *
     * @param transform Options to control which units and values are acceptable.
     * @param forOutput The output options that should be used to determine which
     *  value & unit is best.
     * @return A new range.
     * @see [[Value.normalize]]
     */
    normalize(transform: Transform, forOutput: Output): Range;
    /**
     * Adds this range and a given range (optionally scaled by a factor) together.
     *
     * @param addend The range to add to this instance.
     * @param scale The factor to multiply the addend by when added it to this
     *  instance.
     * @return a new range.
     * @see [[Value.add]]
     */
    add(addend: Range, scale?: number): Range;
    /**
     * Subtracts a given range (optionally scaled by a factor) from this range.
     *
     * @param subtrahend The range to remove from this instance.
     * @param scale The factor to multiply the subtrahend by when subtracting it
     *  from this instance.
     * @return A new range.
     * @see [[Value.sub]]
     */
    sub(subtrahend: Range, scale?: number): Range;
    /**
     * Multiplies this range by a scalar factor.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.scale]]
     */
    scale(scale: number): Range;
    /**
     * Multiplies this range by a scale value.
     *
     * @param scale The amount to multiply the range by.
     * @return A new range.
     * @see [[Value.mul]]
     */
    mul(scale: Value): Range;
    /**
     * Returns a range which is coerced into being represented by fractions if a
     * valid fraction can be determined from the units valid denominators.
     *
     * @return A new range if the minimum and maximum are not fractions, otherwise
     *  the reference to this range is returned.
     * @see [[Value.fractioned]]
     */
    fractioned(): Range;
    /**
     * Returns a range which has any fraction values converted to numbers.
     *
     * @return A new range if the mimimum or maximum are fractions, otherwise the
     *  the reference to this range is returned.
     * @see [[Value.numbered]]
     */
    numbered(): Range;
    /**
     * Converts this range to a string with the given output options taking into
     * account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    output(options?: OutputInput): string;
    /**
     * Creates a fixed range from a given value. A fixed range behaves essentially
     * as a value since the minimum and maximum are equivalent.
     *
     * @param fixed The value to be used as the min and max of the range.
     * @return A new fixed range.
     */
    static fromFixed(fixed: Value): Range;
}
