import { Range } from './Range';
/**
 * The enumeration which decides what value in a range should be used when
 * sorting between ranges with differing minimum and maximum values.
 */
export declare enum SortType {
    /**
     * This value will use the minimum of the ranges to sort by.
     */
    MIN = 0,
    /**
     * This value will use the maximum of the ranges to sort by.
     */
    MAX = 1,
    /**
     * This value will use the average of the ranges to sort by.
     */
    AVERAGE = 2,
}
/**
 * Developer input which can be passed to a Sort constructor or the sort
 * function that uses the [[Sort]] object.
 *
 * @see [[Base.sort]]
 */
export interface SortInput {
    /**
     * @see [[Sort.ascending]]
     */
    ascending?: boolean;
    /**
     * @see [[Sort.type]]
     */
    type?: SortType;
    /**
     * @see [[Sort.classes]]
     */
    classes?: SortClassMap;
}
/**
 * An object which specifies the priority between different class types.
 */
export declare type SortClassMap = {
    /**
     * The priorities are numbers keyed by the class names.
     */
    [className: string]: number;
};
/**
 * A function which takes two ranges and returns a number which discribes the
 * ordering relationship between the two ranges. If a < b then a negative value
 * is returned. If a > b then a positive value is returned. If a = b then zero
 * is returned.
 *
 * @param a The first range.
 * @param b The second range.
 * @return A number which describes the ordering between the two ranges.
 */
export declare type Sorter = (a: Range, b: Range) => number;
/**
 * The class which determines how to sort ranges.
 */
export declare class Sort implements SortInput {
    /**
     * If the ranges should be in ascending order (small values followed by large
     * values). The default value is in descending order.
     */
    ascending: boolean;
    /**
     * How ranges should be compared when the minimum and maximum values differ.
     */
    type: SortType;
    /**
     * This object describes how ranges of different classes should be sorted by
     * given each class a priority. If a class is not defined here the priority
     * assumed is zero.
     */
    classes: SortClassMap;
    /**
     * Creates a new instance of Sort with an optional set of options to override
     * the default values.
     *
     * @param input The options to apply to the new instance.
     */
    constructor(input?: SortInput);
    /**
     * Overrides values in this instance ith ones specified in the input. If class
     * sorting options are specified they are merged into this instance as opposed
     * to a complete overwrite.
     *
     * @param input The values to override.
     * @return The reference to this instance.
     */
    set(input: SortInput): this;
    /**
     * Returns a Sort instance which matches the desired options. If no options
     * are specified the reference to this instance is returned. If the options
     * are already an instance of Sort its returned. If options are specified
     * a new instance is created with the options of this instance, and the given
     * options applied with [[Sort.set]].
     *
     * @param input The options desired.
     * @return An instance of this class which matches the desired options.
     */
    extend(input?: SortInput): Sort;
    /**
     * Returns a function which can sort ranges based on the options in this
     * instance. Comparison is first done by class, and followed by type.
     */
    getSorter(): Sorter;
    /**
     * A sort function between two ranges which look at the range minimums.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    private getMinComparison(a, b);
    /**
     * A sort function between two ranges which look at the range maximums.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    private getMaxComparison(a, b);
    /**
     * A sort function between two ranges which look at the range averages.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    private getAverageComparison(a, b);
    /**
     * A sort function between two ranges which look at the range classes.
     *
     * @param a The first range.
     * @param b The second range.
     * @see [[Sorter]]
     */
    private getClassComparison(a, b);
}
