import { System } from './System';
import { Group } from './Group';
import { Class } from './Class';
import { Range } from './Range';
/**
 * Developer input which can be passed to a Transform constructor or all the
 * various functions that use the [[Transform]] object.
 *
 * @see [[Base.normalize]]
 * @see [[Base.compact]]
 * @see [[Base.expand]]
 * @see [[Base.conversions]]
 * @see [[Base.filter]]
 */
export interface TransformInput {
    /**
     * @see [[Transform.common]]
     */
    common?: boolean;
    /**
     * @see [[Transform.system]]
     */
    system?: System;
    /**
     * @see [[Transform.min]]
     */
    min?: number;
    /**
     * @see [[Transform.max]]
     */
    max?: number;
    /**
     * @see [[Transform.groupless]]
     */
    groupless?: boolean;
    /**
     * @see [[Transform.convertWithMax]]
     */
    convertWithMax?: boolean;
    /**
     * @see [[Transform.onlyUnits]]
     */
    onlyUnits?: string[];
    /**
     * @see [[Transform.notUnits]]
     */
    notUnits?: string[];
    /**
     * @see [[Transform.onlyClasses]]
     */
    onlyClasses?: string[];
    /**
     * @see [[Transform.notClasses]]
     */
    notClasses?: string[];
    /**
     * @see [[Transform.convertUnit]]
     */
    convertUnit?: boolean;
    /**
     * @see [[Transform.convertRate]]
     */
    convertRate?: boolean;
}
/**
 * THe class which controls which units and values are acceptable when
 * transforming a set of ranges.
 *
 * @see [[Base.normalize]]
 * @see [[Base.compact]]
 * @see [[Base.expand]]
 * @see [[Base.conversions]]
 * @see [[Base.filter]]
 */
export declare class Transform implements TransformInput {
    /**
     * The option which determines whether only common or any group are valid.
     * To only include common units this value must be `true` and to include
     * common and uncommon this value must be `false`.
     */
    common: boolean;
    /**
     * The desired system for the transformation.
     */
    system: System;
    /**
     * The mimimum allowed value for the transformation.
     */
    min: number;
    /**
     * The maximum allowed value for the transformation.
     */
    max: number;
    /**
     * Whether the minimum or maximum value of a range is used when producing
     * conversions.
     */
    convertWithMax: boolean;
    /**
     * Whether conversions should convert the main unit.
     */
    convertUnit: boolean;
    /**
     * Whether conversions should convert the rate unit.
     */
    convertRate: boolean;
    /**
     * Whether ranges without units are considered valid for the transformation.
     */
    groupless: boolean;
    /**
     * An array of units that define the valid ranges for a transformation.
     */
    onlyUnits: string[];
    /**
     * An array of units that define the invalid ranges for a transformation.
     */
    notUnits: string[];
    /**
     * An array of class names that define the valid ranges for a transformation.
     */
    onlyClasses: string[];
    /**
     * An array of class names that define the invalid ranges for a transformation.
     */
    notClasses: string[];
    /**
     * Creates a new instance of Transform with an optional set of options to
     * override the default values.
     *
     * @param input The options to apply to the new instance.
     */
    constructor(input?: TransformInput);
    /**
     * Overrides values in this instance with ones specified in input.
     *
     * @param input The values to override.
     * @return The reference to this instance.
     */
    set(input: TransformInput): this;
    /**
     * Returns a Transform instance which matches the desired options. If no
     * options are specified the reference to this instance is returned. If the
     * options are already an instance of Transform its returned. If options are
     * specified a new instance is created with the options of this instance, and
     * the given options applied with [[Transform.set]].
     *
     * @param input The options desired.
     * @return An instance of this class which matches the desired options.
     */
    extend(input?: TransformInput): Transform;
    /**
     * Determines whether the given range is valid according to this instance.
     *
     * @param range The range to test.
     * @return True if the range matches this transform, otherwise false.
     */
    isValidRange(range: Range): boolean;
    /**
     * Determines whether the given group (and optionally a current group) is
     * valid or visible according to this instance.
     *
     * @param group The group to test.
     * @param givenGroup The current group if available.
     * @return True if the group matches this transform, otherwise false.
     */
    isVisibleGroup(group: Group, givenGroup?: Group): boolean;
    /**
     * Determines whether the given group matches the common option on this
     * instance.
     *
     * @param group The group to test.
     * @return True if the group matches the common option, otherwise false.
     */
    isCommonMatch(group: Group): boolean;
    /**
     * Determines whether the given group (and optionally a current group)
     * matches the system option on this instance.
     *
     * @param group The group to test.
     * @param givenGroup The current group if available.
     * @return True if the group matches ths system option, otherwise false.
     */
    isSystemMatch(group: Group, givenGroup?: Group): boolean;
    /**
     * Determines whether the given class matches the classes options on this
     * instance.
     *
     * @param parent The class to test.
     * @return True if the class matches the classes options, otherwise false.
     */
    isClassMatch(parent: Class): boolean;
    /**
     * Determines whether the given group matches the unit options on this
     * instance.
     *
     * @param group The group to test.
     * @return True if the group matches the unit options, otherwise false.
     */
    isUnitMatch(group: Group): boolean;
}
