/**
 * A function which takes user input and a group factor and converts the user
 * input into another string to be parsed by the next translator or the
 * [[Parse]] class.
 *
 * @param x The input to parse.
 * @return The translated input or the original input.
 */
export declare type Translator = (x: string) => string;
/**
 * A function which provides a regular expression match, a group factory, and
 * a constant variable which was provided earlier to run a translation and
 * return the translated value.
 *
 * @param matches The regular expression group matches.
 * @param vars The constant variable passed to [[newRegexTranslator]].
 */
export declare type RegexTranslator = (matches: any[], vars?: any) => string;
/**
 * Creates a [[Translator]] which matches against a regular expression and when
 * the user input matches the regular expression another handler function is
 * called to translate the input. Optionally a constant value can be passed
 * to this function and down to the translator.
 *
 * @param regex The regular expression to match against user input.
 * @param handler The function to call if the input matched the expression.
 * @param vars The constant value to pass to the [[RegexTranslator]].
 * @return A [[Translator]] function.
 */
export declare function newRegexTranslator(regex: RegExp, handler: RegexTranslator, vars?: any): Translator;
/**
 * The class which holds [[Translator]]s to manipulate user input into something
 * more understandable to the [[Parse]] class.
 */
export declare class Translations {
    /**
     * An array of translators that have been registered.
     *
     * @see [[Translations.add]]
     */
    static registered: Translator[];
    /**
     * Adds all translators in the library to be available when parsing.
     */
    static addDefaults(): void;
    /**
     * Adds the given translator to the list of registered translators. This
     * translator will be called last.
     *
     * @param translator The function which translates user input.
  
     */
    static add(translator: Translator): void;
    /**
     * Translates the user input based on the registered translators and returns
     * the final string ready to be parsed.
     *
     * @param input The input to translate.
     * @return The translated string.
     */
    static translate(input: string): string;
    /**
     * A translator which takes a word which represents a number and converts it
     * the respective number.
     *
     * *Examples:*
     * - one [unit]
     * - dozen [unit]
     * - an eleven [unit]
     */
    static NumberWords: Translator;
    /**
     * A translator which takes a word which represents a fraction and multiplies
     * it by the following value.
     *
     * *Examples:*
     * - a third of an acre
     * - half a dozen eggs
     * - a seventh of a mile
     */
    static FractionOfNumber: Translator;
    /**
     * A translator which takes a word which represents a fraction and multiplies
     * it by the following value.
     *
     * *Examples:*
     * - 23 and a half eggs
     * - one and a half acres
     * - 23 and a third
     * - 12 and one fourth
     */
    static AndFraction: Translator;
    /**
     * A translator which takes the amount in parenthesis and moves it out.
     *
     * *Examples:*
     * - (one and a half) acre
     * - (12) tacos
     */
    static Quantity: Translator;
    /**
     * A translator which takes the amount in parenthesis and moves it out.
     *
     * *Examples:*
     * - 1 (6 ounce)
     * - 5 (3 liter)
     */
    static QuantityValue: Translator;
}
