import { Transform } from './Transform';
import { Output, OutputInput } from './Output';
import { Group } from './Group';
/**
 * A class which contains a parsed number or fraction.
 */
export declare class Value {
    /**
     * A value instance which contains invalid numbers.
     */
    static INVALID: Value;
    /**
     * The number parsed or num / den if a fraction was parsed.
     */
    readonly value: number;
    /**
     * The numerator parsed or calculated if this is a fraction, otherwise this
     * will be equal to [[Value.value]].
     */
    readonly num: number;
    /**
     * The denominator parsed or calculated if this is a fraction, otherwise this
     * will be equal to 1.
     */
    readonly den: number;
    /**
     * The group determined based on the unit.
     */
    readonly group: Group;
    /**
     * The unit parsed or chosen to be output to the user.
     */
    readonly unit: string;
    /**
     * The group determined based on the rate.
     */
    readonly rateGroup: Group;
    /**
     * The unit parsed or chosen for rate to be output to the user.
     */
    readonly rate: string;
    /**
     * Creates a new instance of Value given the value, possible numerator and
     * denominator, and the unit and it's group.
     *
     * @param value [[Value.value]]
     * @param num [[Value.num]]
     * @param den [[Value.den]]
     * @param unit [[Value.unit]]
     * @param group [[Value.group]]
     */
    constructor(value: number, num: number, den: number, unit: string, group: Group, rate: string, rateGroup: Group);
    /**
     * Returns true if this value was successfully parsed from some input.
     */
    readonly isValid: boolean;
    /**
     * Returns true if this value is a fraction with a numerator and denoninator.
     */
    readonly isFraction: boolean;
    /**
     * Returns true if this value is a number and not a fraction.
     */
    readonly isDecimal: boolean;
    /**
     * Returns true if this value is zero.
     */
    readonly isZero: boolean;
    /**
     * Returns true if this value is singular.
     *
     * @see [[Functions.isSingular]]
     */
    readonly isSingular: boolean;
    /**
     * Returns true if this value is a rate.
     */
    readonly isRate: boolean;
    /**
     * Returns the number of this value relative to the base unit.
     */
    readonly scaled: number;
    /**
     * Returns the number of this value relative to the first base unit of it's
     * class.
     */
    readonly classScaled: number;
    /**
     * Returns the number which represents the fraction in the value. There may
     * be a difference between this value and the number when the fraction is
     * calculated from the denominators of the group.
     */
    readonly calculated: number;
    /**
     * Returns the whole number for the mixed fraction of this value. If this
     * value is not a fraction 0 is returned.
     */
    readonly mixedWhole: number;
    /**
     * Returns the numerator for the mixed fraction of this value. If this value
     * is not a fraction then the numerator is returned.
     */
    readonly mixedNum: number;
    /**
     * Returns the floor of the number in this value.
     */
    readonly floor: number;
    /**
     * Returns the ceiling of the number in this value.
     */
    readonly ceil: number;
    /**
     * Returns the truncated number in this value taking into account it's sign.
     */
    readonly truncate: number;
    /**
     * Returns the fractional part of the number in this value.
     */
    readonly remainder: number;
    /**
     * Returns the signed distance the number of this value is from the fraction
     * numerator and denominator determined. If this value is not a fraction then
     * this should return zero.
     */
    readonly error: number;
    /**
     * Returns the absolute distance the number of this value is from the fraction
     * numerator and denominator determined. If this value is not a fraction then
     * this should return zero.
     */
    readonly distance: number;
    /**
     * Calculates the scale necessary to switch this value from the current rate
     * to the provided rate.
     *
     * @param rate The rate group.
     * @return The calculated scale.
     */
    getRateScale(rate: Group): number;
    /**
     * Determines whether the given value is compatible with this value to perform
     * operations with.
     *
     * @param other The value to test against.
     * @return True if the given value has compatible unit and rate groups.
     */
    isMatch(other: Value): boolean;
    /**
     * Returns a version of this value with the preferred unit.
     *
     * @return A new value or the reference to this instance if it's groupless.
     * @see [[Group.preferredUnit]]
     */
    preferred(): Value;
    /**
     * Returns a copy of this value.
     *
     * @return A new value.
     */
    copy(): Value;
    /**
     * Returns a value equivalent to zero with the unt and group of this instance.
     *
     * @return A new value.
     */
    zero(): Value;
    /**
     * Returns the truncated version of this value. That's a value where the
     * number is a whole number.
     *
     * @return A new value.
     */
    truncated(): Value;
    /**
     * Returns a version of this value as a fraction.
     *
     * @return A new value or the reference to this instance if it's a fraction.
     */
    fractioned(): Value;
    /**
     * Returns a version of this value as a number.
     *
     * @return A new value or the reference to this instance if it's a number.
     */
    numbered(): Value;
    /**
     * Converts this value to the given group and returns the result.
     *
     * @param to The group to convert to.
     * @param rate The group for the rate.
     * @return The converted value or the number of this value if there's no group.
     */
    convertTo(to: Group, rate?: Group): number;
    /**
     * Converts this value to the given group and returns a new value. The new
     * value will attempted to be converted to a fraction.
     *
     * @param group The group to convert to.
     * @param rate The group for the rate.
     * @return A new value.
     */
    convertToValue(group: Group, rate?: Group): Value;
    /**
     * Determines the available conversions of this value for all groups
     * that are valid for the given transform.
     *
     * @param transform Transform which controls the units and values acceptable.
     * @param reverse Whether to iterate from largest units to smallest units
     *  (`true`), or from smallest to largest (`false`).
     * @param callback The function to invoke for each valid conversion.
     * @param callback.transformed The conversion calculated.
     * @param callback.index The index of the conversion during iteration.
     * @see [[Group.matches]]
     */
    conversions(transform: Transform, reverse: boolean, callback: (transformed: Value, index: number) => void): void;
    /**
     * Returns a value based on this value with the unit that best represents the
     * value. What is best is typically related to the magnitude of the value.
     * Really small and really large values are harder for people to comprehend so
     * the unit which results in the most normal looking value is determined.
     *
     * @param transform Transform which controls the units and values acceptable.
     * @param forOutput The output that may be used so the most normal looking
     *  value can be determined.
     * @return The most normal value found.
     * @see [[Value.conversions]]
     * @see [[Core.isMoreNormal]]
     */
    normalize(transform: Transform, forOutput: Output): Value;
    /**
     * Calculates the sum of this value and the given addend scaled by some
     * factor. This is equivalent to `result = this + (addend * scale)`.
     *
     * @param addend The value to add to this.
     * @param scale The factor to scale the addend by before adding it to this.
     * @return A new instance.
     */
    add(addend: Value, scale?: number): Value;
    /**
     * Calculates the difference between this value and the subtrahend scaled by
     * some factor. This is equivalent to `result = this - (subtrahend * scale)`.
     *
     * @param subtrahend The value to subtract from this.
     * @param scale The factor to scale the subtrahend by before subtraction.
     * @return A new instance.
     */
    sub(subtrahend: Value, scale?: number): Value;
    /**
     * Calculates a new value by multiplying this by a given factor. This is
     * equivalent to `result = this * scale`.
     *
     * @param scale The factor to scale this instance by.
     * @return A new instance.
     */
    scale(scale: number): Value;
    /**
     * Calculates a new value by multiplying this by a given value. This is
     * equivalent to `result = this * scale`.
     *
     * @param scale The value to scale this instance by.
     * @return A new instance.
     */
    mul(scale: Value): Value;
    /**
     * Converts this value to a string with the given output options taking into
     * account the global options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of this instance.
     * @see [[Output]]
     */
    output(options?: OutputInput): string;
    /**
     * Returns the units of this value as a string based on the global output
     * options.
     *
     * @param options The options to override the global output options.
     * @return The string representation of the units of this value.
     * @see [[Output]]
     */
    units(options?: OutputInput): string;
    /**
     * Returns a Value instance which is a number with the optional unit and group.
     *
     * @param value The number.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    static fromNumber(value: number, unit?: string, group?: Group, rate?: string, rateGroup?: Group): Value;
    /**
     * Returns a Value instance which tries to be a fraction given a range of
     * denominators. If the number is already whole or a fraction close
     * enough to the number cannot be found a value which is a number is returned.
     *
     * @param value The number to try to find a fraction for.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @param minDen The starting denominator to inclusively try.
     * @param maxDen The last denominator to inclusively try.
     * @return A new instance.
     */
    static fromNumberWithRange(value: number, unit?: string, group?: Group, minDen?: number, maxDen?: number, rate?: string, rateGroup?: Group): Value;
    /**
     * Returns a Value instance which tries to be a fraction based on the
     * denominators of the group. If a valid fraction could not be found then the
     * instance returned will be a number value. Since a unit is not passed here,
     * the preferred unit of the group is used as the unit of the value.
     *
     * @param value The number to try to find a fraction for.
     * @param group The group for the unit and also the denominators to try.
     * @param rateGroup The group for the rate.
     * @return A new instance.
     */
    static fromNumberForGroup(value: number, group: Group, rateGroup: Group): Value;
    /**
     * Returns a Value instance which tries to be a fraction based on the
     * denominators of the group. If a valid fraction could not be found then the
     * instance returned will be a number value.
     *
     * @param value The number to try to find a fraction for.
     * @param denominators The array of denominators to try.
     * @param unit The unit, if any, of the number.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    static fromNumberWithDenominators(value: number, denominators: number[], unit?: string, group?: Group, rate?: string, rateGroup?: Group): Value;
    /**
     * Returns a Value instance for a given fraction specified by a numerator and
     * denominator.
     *
     * @param num The numerator of the fraction.
     * @param den The denominator of the fraction.
     * @param unit The unit, if any, of the fraction.
     * @param group The group which matches the unit.
     * @return A new instance.
     */
    static fromFraction(num: number, den: number, unit?: string, group?: Group, rate?: string, rateGroup?: Group): Value;
}
